/**
   Copyright (c) 2005,2006,2007,2008 by Conitec Datasysteme GmbH
   Written 2008 by Peer Georgi <georgi@conitec.com>

   License: GPL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#ifndef __cG5AbstractPSCtrl_h
  #define __cG5AbstractPSCtrl_h

#include <string>

class cG5AbstractPSCtrl
  {
  public:
    enum Supply { vls = 0, vgate = 1, vcomp = 2, vcp1 = 3, vcp2 = 4, vcp3 = 5 };
  public:
    cG5AbstractPSCtrl();
    virtual ~cG5AbstractPSCtrl() throw();

    /// \brief Performs a reset of all power supplies.
    virtual bool resetPowerSupplies() = 0;
    /// \brief Returns the resolution of DAC.
    virtual unsigned int dacBits() const = 0;

    /// \brief Set's the voltage (in mV) for given channel.
    virtual bool setVoltage(Supply, unsigned int imV) = 0;
    /// \brief Set's the DAC value of given channel.
    virtual bool setVoltageDAC(Supply, unsigned int iDAC) = 0;
    /// \brief Set's the calibration data for given channel.
    virtual void setSupplyCalibData(Supply, double dGradient, double dOffset) = 0;
    /// \brief Stores the calibration data.
    virtual void storeCalibration() = 0;

    /// \brief Returns the current draws. (Unit: uA)
    virtual bool getCurrentDraws(unsigned int& iIGate, unsigned int& iIComp, unsigned int& iILS, unsigned int& iICP1, unsigned int& iICP2, unsigned int& iICP3) = 0;
    /// \brief Returns the current voltages. (Unit: mV)
    virtual bool getVoltages(unsigned int& iVGate, unsigned int& iVComp, unsigned int& iVLS, unsigned int& iVCP1, unsigned int& iVCP2, unsigned int& iVCP3, unsigned int& iUIn) = 0;
    /// \brief Returns the adc value of a specific voltage.
    virtual unsigned int getVoltageADC(Supply, bool& boOk) = 0;

    /// \brief Returns true if the hardware is capable to read the voltage.
    virtual bool isVoltageReadable(Supply) = 0;
    /// \brief Returns true if the hardware supports reading the current draw.
    virtual bool isCurrentDrawReadable(Supply) = 0;

    /// \brief Returns true if the hardware provides a discharge circuit.
    virtual bool hasDischarge(Supply) const = 0;
    /// \brief Enables / disables the discharge. (Take care! Unappropriate ease of handling may destroy hardware!)
    virtual void setDischarge(Supply, bool) = 0;

    /// \brief Returns the firmware version.
    virtual std::string getFwVersion() const = 0;

    /// \brief Program power controller.
    virtual void setupMCU(const char* pBf, unsigned int iSize) = 0;
  };

#endif // __cG5AbstractPSCtrl_h
