/**
   Copyright (c) 2005,2006,2007,2008 by Conitec Datasysteme GmbH
   Written 2008 by Peer Georgi <georgi@conitec.com>

   License: GPL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#ifndef __cG5LinearSlotHdw_h
  #define __cG5LinearSlotHdw_h

#include <cG5CPDRegSet.h>
#include <cG5PDIOCtrl.h>

/**
  \brief Provides convenient access the DIL48 Socket but requires a valid FPGA hardware that provides Latch and pindriver access.

  See libG5AdapterCtrl/cG5DIL48AdapterMCUHdwInterface for an example.
*/
class cG5LinearSlotHdw
  {
  public:
    enum Direction { dIn, dOut };
  public:
    cG5LinearSlotHdw(cG5PDIOCtrl&);
    ~cG5LinearSlotHdw();

    /// \brief Returns a reference to the CPD registers.
    cG5CPDRegSet& cpdConfig();
    /// \brief Returns a reference to the CPD ctrl. object.
    cG5AbstractCPDCtrl& cpdCtrl();
    /// \brief Configures all CPDs.
    void setupCPDs();

    /// \brief Reads the 64-Bit latch that represents all pins.
    unsigned long long readSocket();
    /// \brief Specifies a new state for all output pins.
    void writeSocket(unsigned long long);
    /// \brief Specifies the pin direction each bit represents a pin. 1 = input, 0 = output.
    void setOutputMask(unsigned long long);

    /// \brief Single pin direction access.
    void setPinDirection(int iPinNumber, Direction); // Pin-Number is zero based //
    /// \brief Single pin output access.
    void setOutputData(int iPinNumber, bool);

    /// \brief Converts the socket-pin number to the correspond CPD / FPGA BigLatch offset.
    static unsigned int pin2Idx(unsigned int iPin);

    /// \brief Returns a pointer that is accesses the FPGA register directly.
    volatile unsigned short* fastOutAccess(unsigned int iPin, unsigned int& iMask);
    /// \brief Returns a pointer that is accesses the FPGA register directly.
    volatile unsigned short* fastInAccess(unsigned int iPin, unsigned int& iMask);

  private:
    cG5PDIOCtrl& mD;
    cG5CPDRegSet mCPDConfig;
    unsigned long long miSocketMask;
    /// \brief Used as shadow for output state.
    unsigned long long miOutputShadow;
    /// \brief Used as direction shadow.
    unsigned long long miDirectionMask;
  };


#endif // __cG5LinearSlotHdw_h
