/**
   Copyright (c) 2005,2006,2007,2008 by Conitec Datasysteme GmbH
   Written 2008 by Peer Georgi <georgi@conitec.com>

   License: GPL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#ifndef __cTinyFs_h
  #define __cTinyFs_h

#include <list>
#include <memory>

class cDFDevice;
class cTinyFsFile;
class cTinyFs
  {
  public:
    cTinyFs(cDFDevice&);
    ~cTinyFs() throw();

   /// \brief Returns a file if exists. If not an exception will be thrown.
   cTinyFsFile& findFile(const std::string& s);
   /// \brief Returns a file if exists.
   cTinyFsFile* hasFile(const std::string& s);

   /// \brief Returns a list of files in file system.
   std::list<cTinyFsFile>& files();
   /// \brief Returns a list of files in file system (const.)
   const std::list<cTinyFsFile>& files() const;
   /// \brief Returns the maximum file size in respect of holes in media and boot-ram.
   unsigned int maxFileSize(const cTinyFsFile&, bool boCheckBootSituation = true) const;

   void writeFAT();

   cDFDevice& device();
   const cDFDevice& device() const;

   // Check for sizes and offsets on media. If boBootTime is true the integrity on boot time will be checked too.
   void checkIntegrity(bool boBootTime);
   // Checks the content integrity. (Checksum tests)
   void checkContentIntegrity(const cTinyFsFile&);

  private:
    std::auto_ptr<std::list<cTinyFsFile> > mount();

  private:
    cDFDevice& mDevice;
    std::auto_ptr<std::list<cTinyFsFile> > mFAT;
    unsigned int miMaxFATItems;
  };





#endif // __cTinyFs_h
