/**
   Copyright (c) 2005,2006,2007,2008 by Conitec Datasysteme GmbH
   Written 2008 by Peer Georgi <georgi@conitec.com>

   License: GPL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#ifndef __cTinyFsFile_h
  #define __cTinyFsFile_h

#include <string>
#include <cGuardedBuffer.h>

class cDFDevice;
class cTinyFs;

class cTinyFsFile
  {
  public:
    cTinyFsFile(cTinyFs&, const std::string& sName, unsigned int iStart, unsigned int iSize, unsigned int iEntryFlags, unsigned int iLoadMemStart);
    ~cTinyFsFile() throw();

    /// \brief Set's the size to zero.
    void clear();
    void write(const char* p, unsigned int i);
    void patch(const char* p, unsigned int iSize);
    void read(char* p, unsigned int iSize) const;

    void write(cGuardedBuffer&);
    std::auto_ptr<cGuardedBuffer> read();

    unsigned int size() const;
    void setName(const std::string&);
    std::string name() const;
    unsigned int mediaStart() const;
    unsigned int loadAddress() const;
    unsigned int attributes() const;

    // attributes //
    bool attrLoadedByBootLoader() const;
    bool attrExecutedByBootLoader() const;
    bool attrBootable() const;
    bool attrAlternateBootOption() const;
    bool attrCompressed() const;

    void setContentCRC16(unsigned short);
    unsigned short contentCRC16() const;

    // max size results on the start address of the following file-item //
    unsigned int maxMediaSize() const;
    unsigned int percentualUsage() const;
    unsigned int bytesLeft() const;

  private:
    cTinyFs& mFs;
    std::string msName;
    unsigned int miMediaStart;
    unsigned int miMediaSize;
    unsigned int miEntryFlags;
    unsigned int miLoadMemStart;
  };


#endif // __cTinyFsFile_h
