/**
   Copyright (c) 2005,2006,2007,2008 by Conitec Datasysteme GmbH
   Written 2008 by Peer Georgi <georgi@conitec.com>

   License: GPL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include <vector>
#include "cGalep5Device.h"
#include <cG5Config.h>
#include <cG5DeviceCtrl.h>
#include <cG5ProductInfo.h>

#include <cG5PDIOCtrl.h>

#include <cG5AbstractPSCtrl.h>
#include <cG5PSCtrl.h>
#include <cG5AbstractButtonCtrl.h>

#include <cDriverAttribute.h>
#include "piDBG.h"
#include <cG5SDCardCtrl.h>

#ifndef __G5FREEAPI__
  #include <cG5AdapterModuleCtrl.h>
  #include <cG5PMUCtrl.h>
#endif

class cG5DButtonCtrl : public cG5AbstractButtonCtrl
  {
  public:
    cG5DButtonCtrl()
      : mAttr("/sys/bus/platform/drivers/galep5-ui/binButton")
        { }

    virtual ~cG5DButtonCtrl() throw()
      { }

    virtual unsigned int shortPress()
      { return m.miNumPressEvents; }
    
    virtual unsigned int longPress()
      { return m.miNumLongPressEvents; }
    
    virtual unsigned int veryLongPress()
      { return m.miNumVeryLongPressEvents; }
    
    virtual bool isDown()
      { return m.miIsPressed; }

    virtual void update()
      {
        mAttr.read((char*)&m, sizeof(m));
      }

  private:
    struct tButtonState
      {
        unsigned int miIsPressed;
        unsigned int miLastPressTime;
        unsigned int miNumPressEvents;
        unsigned int miNumLongPressEvents;
        unsigned int miNumVeryLongPressEvents;
        unsigned int miZero;
      };

    tButtonState m;
    cDriverAttribute mAttr;
  };






class cGalep5Device::cGalep5DeviceImpl
  {
  public:
    cGalep5DeviceImpl(cGalep5Device& d)
      {
        // Obtain information about the device. //
        mpConfig.reset(new cG5Config());

        // Create a device controller. //
        mpDeviceCtrl.reset(new cG5DeviceCtrl(*mpConfig));

        // Create accessors for FPGAs and CPDs //
         // We keep a list of NumSlots+1 PDIOCtrl objects. The first is the zero page accessor. //
        unsigned int iNumPDsInstalled = mpConfig->productInfo().getNumPDsInstalled();
        if (iNumPDsInstalled)
          for (unsigned int i = 0; i < mpConfig->productInfo().getNumPDsInstalled()+1; ++i)
            mVctPDIO.insert(mVctPDIO.end(), new cG5PDIOCtrl(i, mpConfig->productInfo().productType(), mpConfig->productInfo().fpgaBaseMappingAddr()));

        // HACK (If no fpga pcb attached) //
        //mVctPDIO.insert(mVctPDIO.end(), new cG5PDIOCtrl(0, mpConfig->productInfo().productType(), mpConfig->productInfo().fpgaBaseMappingAddr()));
        //mVctPDIO.insert(mVctPDIO.end(), new cG5PDIOCtrl(0, mpConfig->productInfo().productType(), mpConfig->productInfo().fpgaBaseMappingAddr()));

        cG5PDIOCtrl* pSlot1 = 0;
        if (mVctPDIO.size())
          pSlot1 = mVctPDIO[0];

        // Create accessors for power supply. //
        mpPSCtrl.reset(new cG5PSCtrl(pSlot1, mpConfig->productInfo().productType()));

       // At this time only the Galep5M provides a PMU.
#ifndef __G5FREEAPI__       
       if (mpConfig->productInfo().productType() == cG5ProductInfo::Galep5M)
         mpPMUCtrl.reset(new cG5PMUCtrl(d));
#endif

       if (mpConfig->productInfo().productType() == cG5ProductInfo::Galep5Pro)
         mpButtonCtrl.reset(new cG5DButtonCtrl());
      }

    ~cGalep5DeviceImpl() throw()
      {
        for (unsigned int i = 0; i < mVctPDIO.size(); ++i)
          delete mVctPDIO[i];
      }

    cG5AdapterModuleCtrl& adapterModuleCtrl(cGalep5Device& d)
      {
#ifdef __G5FREEAPI__
        throw std::string("Adapter modules not suported yet.");
#else        
        if (mpAdapterModuleCtrl.get())
          return *mpAdapterModuleCtrl;

          cG5PDIOCtrl* pSlot1 = 0;
          cG5PDIOCtrl* pSlot2 = 0;
          if (mVctPDIO.size() > 1)
            pSlot1 = mVctPDIO[1];
          if (mVctPDIO.size() > 2)
            pSlot2 = mVctPDIO[2];
          mpAdapterModuleCtrl.reset(new cG5AdapterModuleCtrl(d, pSlot1, pSlot2, *mpConfig));
          return *mpAdapterModuleCtrl;
#endif
      }

  public:
    std::auto_ptr<cG5Config> mpConfig;
    /// \brief The list of installed PDIO slots.
    std::vector<cG5PDIOCtrl*> mVctPDIO;
    std::auto_ptr<cG5PSCtrl> mpPSCtrl;
    std::auto_ptr<cG5DeviceCtrl> mpDeviceCtrl;
#ifndef __G5FREEAPI__
    std::auto_ptr<cG5AdapterModuleCtrl> mpAdapterModuleCtrl;
    std::auto_ptr<cG5PMUCtrl> mpPMUCtrl;
#endif
    std::auto_ptr<cG5SDCardCtrl> mpSDCardCtrl;
    std::auto_ptr<cG5AbstractButtonCtrl> mpButtonCtrl;
  };


cGalep5Device::cGalep5Device()
  : mpImpl(0),
    miFPGAMapping(0), miLockMask(0)
  {
    mpImpl.reset(new cGalep5DeviceImpl(*this));
    miFPGAMapping = config().productInfo().fpgaBaseMappingAddr();
  }

cGalep5Device::~cGalep5Device() throw()
  {
  }

cG5Config& cGalep5Device::config()
  { return *mpImpl->mpConfig; }

cG5AbstractPSCtrl& cGalep5Device::psCtrl()
  { return mpImpl->mpPSCtrl->psCtrl(); }

cG5AdapterModuleCtrl& cGalep5Device::adapterModuleCtrl()
  {
    return mpImpl->adapterModuleCtrl(*this);
  }

cG5DeviceCtrl& cGalep5Device::deviceCtrl()
  { return *mpImpl->mpDeviceCtrl; }


bool cGalep5Device::isSDCardInstalled() const
  { return (mpImpl->mpSDCardCtrl.get() != 0); }

cG5SDCardCtrl& cGalep5Device::sdCardCtrl()
  {
    if (!(mpImpl->mpSDCardCtrl.get()))
      throw std::string("Galep5Device: no sdcard ctrl object");
    return *mpImpl->mpSDCardCtrl;
  }

bool cGalep5Device::isPMUInstalled() const
  {
#ifdef __G5FREEAPI__
    return false;
#else    
    return (mpImpl->mpPMUCtrl.get() != 0);
#endif
  }

cG5PMUCtrl& cGalep5Device::pmuCtrl()
  {
#ifdef __G5FREEAPI__
    throw std::string("PMU Ctrl not supported.");
#else
    if (!(mpImpl->mpPMUCtrl.get()))
      throw std::string("Galep5Device: no PMU ctrl object");
    return *mpImpl->mpPMUCtrl;
#endif
  }



bool cGalep5Device::hasButton(unsigned int) const
  { return mpImpl->mpButtonCtrl.get() != 0; }

cG5AbstractButtonCtrl& cGalep5Device::button(unsigned int)
  {
    if (!(mpImpl->mpButtonCtrl.get()))
      throw std::string("Galep5Device: no buttons available");
    return *mpImpl->mpButtonCtrl;
  }


unsigned int cGalep5Device::numInstalledPDIOs() const
  {
    return mpImpl->mVctPDIO.size();
  }

cG5PDIOCtrl& cGalep5Device::pdioCtrl(unsigned int iIndex)
  {
    if (iIndex >= numInstalledPDIOs())
      throw std::string("Galep5Device: invalid pdio / slot index");
    return *mpImpl->mVctPDIO[iIndex];
  }

// Defines in "mcu-direct.S" //
extern "C" void mcu_wait_bus_cycles(unsigned int, unsigned int);


void cGalep5Device::waitBusCycles(unsigned int i)
  {
    mcu_wait_bus_cycles(i, miFPGAMapping);
  }

#include <cAT91RM9200DIO.h>
void cGalep5Device::lockInterrupts()
  {
    miLockMask = cAT91RM9200DIO::cAIC::getEnabled();
    cAT91RM9200DIO::cAIC::disableAll();
  }

void cGalep5Device::unlockInterrupts()
  {
    cAT91RM9200DIO::cAIC::enable(miLockMask);
  }
