/**
   Copyright (c) 2005,2006,2007,2008 by Conitec Datasysteme GmbH
   Written 2008 by Peer Georgi <georgi@conitec.com>

   License: GPL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include "cG5DeviceCtrl.h"
#include <cG5Config.h>
#include <cG5FirmwareCtrl.h>

#include "releases/4.x/cG5Fw4xStateCtrl.h"

/**
  Private implementation.
*/
class cG5DeviceCtrl::cG5DeviceCtrlImpl
  {
  public:
    cG5DeviceCtrlImpl(const cG5Config& config)
      {
        // We have to choose a state controller according the firmware version //
        std::string sFwVersion = config.firmware().varPartVersion();
        mpStateCtrl.reset(new cG5Fw4xStateCtrl(config));


        if (!(mpStateCtrl.get()))
          throw std::string("cG5DeviceCtrl: Unknown firmware - no state controller");
      }

    ~cG5DeviceCtrlImpl() throw()
      {
      }

   public:
     std::auto_ptr<cG5AbstractStateCtrl> mpStateCtrl;
  };








cG5DeviceCtrl::cG5DeviceCtrl(const cG5Config& config)
  {
    mpImpl = std::auto_ptr<cG5DeviceCtrlImpl>(new cG5DeviceCtrlImpl(config));
  }

cG5DeviceCtrl::~cG5DeviceCtrl() throw()
  { /* auto_ptr destroys implementation */ }

const cG5AbstractStateCtrl& cG5DeviceCtrl::state() const
  { return *mpImpl->mpStateCtrl; }

cG5AbstractStateCtrl& cG5DeviceCtrl::stateCtrl()
  { return *mpImpl->mpStateCtrl; }
