/**
   Copyright (c) 2005,2006,2007,2008 by Conitec Datasysteme GmbH
   Written 2008 by Peer Georgi <georgi@conitec.com>

   License: GPL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#ifndef __cG5Fw4xStateCtrl_h
  #define __cG5Fw4xStateCtrl_h

#include <memory>
#include <cG5AbstractStateCtrl.h>

class cG5Config;

/**
  \brief This class represents the device state on early firmware versions (below 5.0).
  On those firmware versions the device state will not kept in kernel! The only sense for this class is to
  drive the Ready / Busy LEDs while programming.
  SD-Card, User Interaction, Errormanagement, Progressbar are not supported yet.
*/
class cG5Fw4xStateCtrl : public cG5AbstractStateCtrl
  {
  public:
    cG5Fw4xStateCtrl(const cG5Config&);
    virtual ~cG5Fw4xStateCtrl() throw();

    virtual DeviceState state() const;
    virtual bool readyForJob() const;

    virtual void setDeviceState(DeviceState);

    virtual void setProgress(unsigned int);
    virtual unsigned int progress() const;

    virtual void setDeviceError(unsigned int iDiagCode);
    virtual void unsetDeviceError();
    virtual bool isDeviceError() const;
    virtual unsigned int deviceErrorCode() const;

    virtual void showSuccess();
    virtual void showFailure();
    virtual void showIdent();

    virtual void buttonGreen(bool);
    virtual void buttonRed(bool);

    virtual void claimDevice(const std::string& sInterfaceName, const std::string& sRemoteId, bool boForce);
    virtual void unclaimDevice();
    virtual std::string claimedInterface() const;
    virtual std::string claimedRemoteId() const;

    virtual void setLED(char);

  private:
    class cG5Fw4xStateCtrlImpl;
    std::auto_ptr<cG5Fw4xStateCtrlImpl> mpImpl;
  };

#endif // __cG5Fw4xStateCtrl_h
