/**
   Copyright (c) 2005,2006,2007,2008 by Conitec Datasysteme GmbH
   Written 2008 by Peer Georgi <georgi@conitec.com>

   License: GPL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include "cG5AbstractCPDCtrl.h"
#include "cG5CPDRegSet.h"
#include <unistd.h>


// Legt fest, ob das FPGA die CPD Register selbsstndig schiften soll.
//#define G5CPD_ACCESS_FPGA_SHIFT

/*
  CPD-OPC Mapping:
    Die OPC-Indices von cG5CPDRegSet sind rein logischer Natur. OPC[0] entspricht also P0 an einer
    Galep5-P Pintreiber-Platine.
    Beim Galep5-P sind die Pintreiberplatinen (max. 6) identisch, sodass auch die OPC-Zuordnungen fr
    jeden Slot identisch sind. Dies ist beim Galep5-D und beim Galep5-M nicht der Fall.

    Im brigen kann die Zahl der installierten Pintreiber pro RegSet variieren.
    (6 beim Galep5-P und Galep5-D. 5 beim Galep5-M)

  Lsung:
    Die Klasse cG5CPDRegSet verwendet keinerlei Mapping. (Sie kann die Umrechnung
    nicht durchfhren, da ein Exemplar von cG5CPDRegSet keine Information ber den Slot hat, auf welchen
    sie evtl. angewendet wird.)

    cG5AbstractCPDCtrl macht im Falle des Galep5-D (u. Galep5-M) ein zustzliches Mapping in Abh. des eingestellten Slots.
*/


cG5AbstractCPDCtrl::cG5AbstractCPDCtrl(cG5FPGACtrl& fpgaCtrl)
  : mFPGACtrl(fpgaCtrl), miDataReadBack(0)
  {
  }

cG5AbstractCPDCtrl::~cG5AbstractCPDCtrl() throw()
  {
  }

/**
  Liest das derzeitige Registerset aus.
  Dabei wird zunchst ein Registerset bestehend aus 60 Nullen herausgeschoben
  und dabei das aktive Registerset ausgelesen.
  Danach wird das ausgelesene Registerset herausgeschoben.
  <br><b>Achtung!</b><br>
  Aufgrund der Tatsache, dass das ausgelesene Registerset danach hineingeschrieben wird,
  kann, im Fehlerfalle, das Registerset unbrauchbar werden!<br>
  Im Fehlerfalle ist der aktuelle Registersatz <b>unbrauchbar</b> und das Ergebnis in
  aDst undefiniert!
*/
bool cG5AbstractCPDCtrl::readRegisterSet(cG5AbstractCPDCtrl::regbank iCRS, cG5CPDRegSet& aDst)
  {
    cG5CPDRegSet aTmp;
    if (!(writeRegisterSet(iCRS, aTmp, &aDst)))
      return (false);
    return (writeRegisterSet(iCRS, aDst));
  }

/**
  Schreibt ein Registerset in das CPD.
  <b>Wenn</b> mit pReadBack ein Zeiger auf ein gltiges cG5CPDRegSet-Exemplar bergeben wurde,
  wird dieses mit den <b>vorher</b> aktiven Registerdaten gefllt.
  Rckgabewert:
    + true : erfolg
    + false: schreib-Timeout \Ref{waitTxReady}().
  Array Index 0 entspricht OPC0 des LETZTEN CPDs.
*/
#include <iostream>
bool cG5AbstractCPDCtrl::writeRegisterSet(cG5AbstractCPDCtrl::regbank iCRS, const cG5CPDRegSet& aSrc, cG5CPDRegSet* pReadBack)
  {
    unsigned int iIdx = mFPGACtrl.index()-1;
    const unsigned char* pOPCMappingTable = mappingVector(iIdx);
    setCRS(iCRS == cG5AbstractCPDCtrl::rb_crs1);
    for (unsigned int i = 0; i < numCPDs()*10; i++)
      {
        unsigned int iMappedIndex = pOPCMappingTable[i];
//        if (aSrc.getReg(iMappedIndex))
//          std::cout << "Mapping index: " << i << " to index " << iMappedIndex << " value " << aSrc.getReg(iMappedIndex) << std::endl;
        if (!(sendCPDWord(aSrc.getReg(iMappedIndex))))
          return (false);
        if (pReadBack)
          {
            unsigned short iValue = 0;
            if (!(readCPDWord(iValue)))
              return (false);
            pReadBack->setReg(iMappedIndex, iValue);
          }
      }
    if (!(pReadBack))
      pulseCPDStrobe();
    return (true);
  }

/**
  Setzt das Strobe-Signal
*/
void cG5AbstractCPDCtrl::setStrobe(bool bo)
  {
    unsigned short wReg = mFPGACtrl.read16(G5_FPGA_CPD_CTRL_ADDR);
    if(bo)
      mFPGACtrl.write16(G5_FPGA_CPD_CTRL_ADDR, wReg | (1 << 15));
    else
      mFPGACtrl.write16(G5_FPGA_CPD_CTRL_ADDR, wReg &= ~(1 << 15));
  }

/**
  Erzeugt einen Strobe-Impuls.
*/
void cG5AbstractCPDCtrl::pulseCPDStrobe()
  {
#ifdef G5CPD_ACCESS_FPGA_SHIFT
    mFPGACtrl.write16(G5_FPGA_CPD_STROBR_ADDR, 0x0); // value is not important //
#else
    unsigned short wReg = mFPGACtrl.read16(G5_FPGA_CPD_CTRL_ADDR);
    mFPGACtrl.write16(G5_FPGA_CPD_CTRL_ADDR, wReg | (1 << 15));
    for (unsigned int iii = 0; iii < 100; iii++);
    mFPGACtrl.write16(G5_FPGA_CPD_CTRL_ADDR, wReg &= ~(1 << 15));
#endif
  }


// Methoden, welche das FPGA-Shiftregister verwenden
#ifdef G5CPD_ACCESS_FPGA_SHIFT
/**
  Sendet ein Datenwort zum CPD (12-Bits).
  Rckgabewert:
    true  - erfolg
    false - timeout (\Ref{waitTxReady}())
  <br><b>Hinweis</b><br>
  Vor evtl. nachfolgender Leseoperation solle mit
  waitTxReady() geprft werden, wann der Schreibvorgang
  beendet ist!
*/
bool cG5AbstractCPDCtrl::sendCPDWord(unsigned short wData)
  {
    if (!(waitTxReady()))
      return (false);
    write16(G5_FPGA_CPD_SHIFTREG_ADDR, wData);
    return (true);
  }

/**
  Liest vom Shiftregister.
*/
bool cG5AbstractCPDCtrl::readCPDWord(unsigned short& wData)
  {
    if (!(waitTxReady())) return (false);
    wData = read16(G5_FPGA_CPD_SHIFTREG_ADDR);
    return (true);
  }

#else
// Diese Methoden verwenden direkte Portzugriffe. (Ohne FPGA-Shiftregisterfunktionen)

/**
  Sendet ein Datenwort zum CPD (12-Bits).
  Rckgabewert: true
*/
bool cG5AbstractCPDCtrl::sendCPDWord(unsigned short wData)
  {
    unsigned short wReg = 0;
    unsigned short wDataReadBack = 0;
    for (unsigned int i = 0; i < 12; i++)
      {
        wReg = mFPGACtrl.read16(G5_FPGA_CPD_CTRL_ADDR);
        // get SDI before rising edge of clk //
        if (((wReg & (1 << 8)) != 0))
          wDataReadBack |= (1 << i);

        // keep clk -> low //
        wReg &= ~(1 << 14);
        if (wData & 1)
          wReg |= (1 << 13);
        else
          wReg &= ~(1 << 13);
        // write data in dependency of wData::i and clock = low //
        mFPGACtrl.write16(G5_FPGA_CPD_CTRL_ADDR, wReg);
        // setup time before rising edge tSsdi //
#if 0
        for (unsigned int iii = 0; iii < 100; iii++);
#endif
        // clk -> high //
        wReg |= (1 << 14);
        mFPGACtrl.write16(G5_FPGA_CPD_CTRL_ADDR, wReg);
        // hold time before falling edge //
#if 0
        for (unsigned int iii = 0; iii < 100; iii++);
#endif
        // clk -> low //
        wReg &= ~(1 << 14);
        mFPGACtrl.write16(G5_FPGA_CPD_CTRL_ADDR, wReg);
        // wait manifestation time //
#if 0
        for (unsigned int iii = 0; iii < 100; iii++);
#endif
        // next bit to write //
        wData >>= 1;
      }
    miDataReadBack = wDataReadBack;
    return (true);
  }

/**
  Liest vom Shiftregister.
  Rckgabewert: true
*/
bool cG5AbstractCPDCtrl::readCPDWord(unsigned short& wData)
  {
    wData = miDataReadBack;
    return (true);
  }


#endif

/**
  Wartet, bis der Transferpuffer leer ist.
  Rckgabewert:
    true:  Der Transferpuffer ist leer geworden
    false: Der Transferpuffer wird wohl nie mehr leer!
*/
bool cG5AbstractCPDCtrl::waitTxReady()
  {
    unsigned int iTimeOut = 1024*1024; // should be long enough //
    while (iTimeOut--)
      {
        if (!(mFPGACtrl.read16(G5_FPGA_CPD_CTRL_ADDR) & 0x80))
          return (true);
      }
    return (false);
  }

void cG5AbstractCPDCtrl::setCRS(bool bo)
  {
    mFPGACtrl.write16(G5_FPGA_CPD_CRS_ADDR, (bo) ? (1) : (0));
  }

void cG5AbstractCPDCtrl::setDIR(bool bo)
  {
    mFPGACtrl.write16(G5_FPGA_CPD_CTRL_ADDR, (bo) ? (mFPGACtrl.read16(G5_FPGA_CPD_CTRL_ADDR) | CPD_CTRL_DIR) : (mFPGACtrl.read16(G5_FPGA_CPD_CTRL_ADDR) &~ CPD_CTRL_DIR));
  }

void cG5AbstractCPDCtrl::setNOEGlobal(bool bo)
  {
    mFPGACtrl.write16(G5_FPGA_CPD_CTRL_ADDR, (bo) ? (mFPGACtrl.read16(G5_FPGA_CPD_CTRL_ADDR) | CPD_CTRL_NOE_GLOBAL) : (mFPGACtrl.read16(G5_FPGA_CPD_CTRL_ADDR) &~ CPD_CTRL_NOE_GLOBAL));
  }

void cG5AbstractCPDCtrl::setNOEVCP1(bool bo)
  {
    mFPGACtrl.write16(G5_FPGA_CPD_CTRL_ADDR, (bo) ? (mFPGACtrl.read16(G5_FPGA_CPD_CTRL_ADDR) | CPD_CTRL_NOE_VCP1) : (mFPGACtrl.read16(G5_FPGA_CPD_CTRL_ADDR) &~ CPD_CTRL_NOE_VCP1));
  }

void cG5AbstractCPDCtrl::setNOEVCP2(bool bo)
  {
    mFPGACtrl.write16(G5_FPGA_CPD_CTRL_ADDR, (bo) ? (mFPGACtrl.read16(G5_FPGA_CPD_CTRL_ADDR) | CPD_CTRL_NOE_VCP2) : (mFPGACtrl.read16(G5_FPGA_CPD_CTRL_ADDR) &~ CPD_CTRL_NOE_VCP2));
  }

void cG5AbstractCPDCtrl::setNOEVCP3(bool bo)
  {
    mFPGACtrl.write16(G5_FPGA_CPD_CTRL_ADDR, (bo) ? (mFPGACtrl.read16(G5_FPGA_CPD_CTRL_ADDR) | CPD_CTRL_NOE_VCP3) : (mFPGACtrl.read16(G5_FPGA_CPD_CTRL_ADDR) &~ CPD_CTRL_NOE_VCP3));
  }

void cG5AbstractCPDCtrl::setNOEGND(bool bo)
  {
    mFPGACtrl.write16(G5_FPGA_CPD_CTRL_ADDR, (bo) ? (mFPGACtrl.read16(G5_FPGA_CPD_CTRL_ADDR) | CPD_CTRL_NOE_GND) : (mFPGACtrl.read16(G5_FPGA_CPD_CTRL_ADDR) &~ CPD_CTRL_NOE_GND));
  }

bool cG5AbstractCPDCtrl::getDIR()
  {
    return (mFPGACtrl.read16(G5_FPGA_CPD_CTRL_ADDR) & CPD_CTRL_DIR);
  }

bool cG5AbstractCPDCtrl::getNOEGlobal()
  {
    return (mFPGACtrl.read16(G5_FPGA_CPD_CTRL_ADDR) & CPD_CTRL_NOE_GLOBAL);
  }

bool cG5AbstractCPDCtrl::getNOEVCP1()
  {
    return (mFPGACtrl.read16(G5_FPGA_CPD_CTRL_ADDR) & CPD_CTRL_NOE_VCP1);
  }

bool cG5AbstractCPDCtrl::getNOEVCP2()
  {
    return (mFPGACtrl.read16(G5_FPGA_CPD_CTRL_ADDR) & CPD_CTRL_NOE_VCP2);
  }

bool cG5AbstractCPDCtrl::getNOEVCP3()
  {
    return (mFPGACtrl.read16(G5_FPGA_CPD_CTRL_ADDR) & CPD_CTRL_NOE_VCP3);
  }

bool cG5AbstractCPDCtrl::getNOEGND()
  {
    return (mFPGACtrl.read16(G5_FPGA_CPD_CTRL_ADDR) & CPD_CTRL_NOE_GND);
  }

bool cG5AbstractCPDCtrl::getCRS()
  {
    return (mFPGACtrl.read16(G5_FPGA_CPD_CRS_ADDR) & 1);
  }
