/**
   Copyright (c) 2005,2006,2007,2008 by Conitec Datasysteme GmbH
   Written 2008 by Peer Georgi <georgi@conitec.com>

   License: GPL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#ifndef __cG5AbstractCPDCtrl_h
  #define __cG5AbstractCPDCtrl_h

#include "cG5FPGACtrl.h"
class cG5CPDRegSet;

class cG5AbstractCPDCtrl
  {
  public:
    enum regbank { rb_crs0, rb_crs1 };
  public:
    /// Fr iFPGAIdx siehe cG5FPGACtrl
    cG5AbstractCPDCtrl(cG5FPGACtrl&);
    virtual ~cG5AbstractCPDCtrl() throw();

    /// \brief Returns the number of CPDs (chips) installed.
    virtual unsigned int numCPDs() const = 0;
    /// \brief Returns the number of used pindrivers. (48 for galep5M).
    virtual unsigned int numUsedPinDrivers() const = 0;
    /// \brief Reutnrs a mapping vector (hardware dependend) for requested cpd.
    virtual const unsigned char* mappingVector(unsigned int) const = 0;

    /// \brief Writes the entire cpd register set.
    bool writeRegisterSet(regbank, const cG5CPDRegSet&, cG5CPDRegSet* pReadBack = 0);
    /**
      \brief Reads out the entire cpd register set.
      On successfully operation this method returns true and the CPD state has not been changed.
      After errors the CPD - state is unknown.
    */
    bool readRegisterSet(regbank, cG5CPDRegSet&);

  public:
    void pulseCPDStrobe();
    void setStrobe(bool);
    void setCRS(bool);
    void setDIR(bool = true);
    void setNOEGlobal(bool = true);
    void setNOEVCP1(bool = true);
    void setNOEVCP2(bool = true);
    void setNOEVCP3(bool = true);
    void setNOEGND(bool = true);

    bool getDIR();
    bool getNOEGlobal();
    bool getNOEVCP1();
    bool getNOEVCP2();
    bool getNOEVCP3();
    bool getNOEGND();
    bool getCRS();

  private:
    bool sendCPDWord(unsigned short wData);
    bool readCPDWord(unsigned short& wData);
    bool waitTxReady();

  private:
    /// \brief Our fpga handle.
    cG5FPGACtrl& mFPGACtrl;
    /// Nur, wenn das FPGA shift Register NICHT verwendet wird, NACH sendCPDWord gltig.
    unsigned short miDataReadBack;
  };

#endif // __cG5AbstractCPDCtrl_h
