/**
   Copyright (c) 2005,2006,2007,2008 by Conitec Datasysteme GmbH
   Written 2008 by Peer Georgi <georgi@conitec.com>

   License: GPL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include <string>
#include "cG5CPDRegSet.h"

cG5CPDRegSet::cG5CPDRegSet()
  {
    for (unsigned int i = 0; i < G5_CPD_REGISTER_SET_ITEMS; i++)
      {
        // initialize array //
        miRegData[i] = G5_CPD_REGISTER_INITIALIZE_VALUE;
      }
  }

cG5CPDRegSet::~cG5CPDRegSet()
  {
  }

/**
  iIdx [0..59] Bezieht sich auf das Array (nicht gleich OPCIndex(!))
  iData 12-Bit value (OPC config register)
*/
void cG5CPDRegSet::setReg(unsigned int iIdx, unsigned int iData)
  {
    if (iIdx < G5_CPD_REGISTER_SET_ITEMS)
      miRegData[iIdx] = iData;
  }

/**
  iIdx [0..59] Bezieht sich auf das Array (nicht gleich OPCIndex(!))
  Return value:
    12 bit OPC register value
*/
unsigned int cG5CPDRegSet::getReg(unsigned int iIdx) const
  {
    if (iIdx < G5_CPD_REGISTER_SET_ITEMS)
      return (miRegData[iIdx]);
    return (0);
  }

/**
  Rechnet die bergebene Pin-Nummer (siehe Schaltplan) in den Array-Index
  des OPC Arrays um.
  bergeben wird 0 fr P0 bis 59 fr P59.
  Rckgabewert: OPC-Array Index
*/
unsigned int cG5CPDRegSet::getRegIndexFromPin(unsigned int iPin)
  {
    if (iPin > 59)
      throw std::string ("cG5CPDRegSet: Pin out of range!");
    return (iPin);
  }

void cG5CPDRegSet::clear()
  {
    for (unsigned int i = 0; i < G5_CPD_REGISTER_SET_ITEMS; ++i)
      miRegData[i] = 0;
  }
