/**
   Copyright (c) 2005,2006,2007,2008 by Conitec Datasysteme GmbH
   Written 2008 by Peer Georgi <georgi@conitec.com>

   License: GPL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#ifndef __cG5FPGACtrl_h
  #define __cG5FPGACtrl_h

#include <string>

// << cpd control >>
#define G5_FPGA_CPD_BASE                (0x10)
#define G5_FPGA_CPD_SHIFTREG_ADDR       (G5_FPGA_CPD_BASE)
#define G5_FPGA_CPD_CTRL_ADDR           (G5_FPGA_CPD_BASE+1)
#define G5_FPGA_CPD_CRS_ADDR            (G5_FPGA_CPD_BASE+2)
#define G5_FPGA_CPD_STROBR_ADDR         (G5_FPGA_CPD_BASE+3)

#define G5_FPGA_CPD_PORT_BASE           (0x64)

// test hardware //
// [
#define G5_FPGA_DATA_PORT_ADDRESS       (G5_FPGA_CPD_PORT_BASE)
#define G5_FPGA_DATA_FIXED_ADDRESS      (G5_FPGA_CPD_PORT_BASE+1)
#define G5_FPGA_DATA_DIR_ADDRESS        (G5_FPGA_CPD_PORT_BASE+2)
// ]

#define G5_FPGA_GET_CPD_PORT_OFFS(A)    (A*3)


#define CPD_CTRL_DIR                    (1 << 0)
#define CPD_CTRL_NOE_GLOBAL             (1 << 1)
#define CPD_CTRL_NOE_VCP1               (1 << 2)
#define CPD_CTRL_NOE_VCP2               (1 << 3)
#define CPD_CTRL_NOE_VCP3               (1 << 4)
#define CPD_CTRL_NOE_GND                (1 << 5)


// << common interface for all hardwares >> //
#define G5_FPGA_LATCH_BASE              (0x20)
#define G5_FPGA_DATA_LATCH_ADDRESS(a)   (G5_FPGA_LATCH_BASE+(4*a))

#define G5_FPGA_DATALINES_WRITE_16      (0x43)
#define G5_FPGA_DATALINES_READ_16       (0x42)
#define G5_FPGA_ADDRESSLINES_32_LSB     (0x40)
#define G5_FPGA_ADDRESSLINES_32_MSB     (0x41)
#define G5_FPGA_ADDRESSLINES_32_INC     (0x44)

// !!! Busy regiter : for old design !!!
// ----------------------------------------------

// mux. register (readback channel ) //
#define G5_FPGA_READBACK_SELECT         (0x30)
#define G5_FPGA_READBACK_DATA           (0x31)
#define G5_FPGA_READBACK_DATA_MUX_BIT   (1 << 15)

// !!! Busy regiter : for new design !!!
// ----------------------------------------------

#define G5_FPGA_GET_BUSY_ADR            (0x30)

class cG5PDIOCtrl;

class  cG5FPGACtrl
  {
  friend class cG5PDIOCtrl;
  public:
    virtual ~cG5FPGACtrl();

    bool isMappingValid() const;
    unsigned int index() const;

    void write16_volatile(unsigned int iOffs, unsigned short);
    void write16(unsigned int iOffs, unsigned short);
    unsigned short read16_volatile(unsigned int iOffs);
    void write16_asm(unsigned int iOffs, unsigned short);
    unsigned short read16(unsigned int iOffs);

    /// \brief Read 32-bit value from given (BEWARE! iOffs is a 32-bit word address!)
    unsigned int read32(unsigned int iOffs);
    /// \brief Write a 32 bit value to given ((BEWARE! iOffs is a 32-bit word address!)
    void write32(unsigned int iOffs, unsigned int);

    static unsigned int getFPGABaseMapping();
    static void loadHdw(unsigned int iFPGAIndex, const char* pData, unsigned int iSize, std::string& sReturn);

    volatile unsigned short* mpIOSpace;

  private:
    unsigned short* getFPGABase(unsigned int iIdx);

  private:
    /// FPGA-Index beginnt derzeit bei 1, da das Erste FPGA bei 0x100 beginnt (nicht bei 0)
    cG5FPGACtrl(unsigned int iFPGAIndex, unsigned int iBaseMappingAddr);

  private:
    unsigned int miIndex;
    unsigned int miFPGABaseMapping;
  };


#endif // __cG5FPGACtrl_h
