/**
   Copyright (c) 2005,2006,2007,2008 by Conitec Datasysteme GmbH
   Written 2008 by Peer Georgi <georgi@conitec.com>

   License: GPL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include "cG5LinearSlotHdw.h"
#include "cG5AbstractCPDCtrl.h"


// From source of G5Admin (see pd-map.inl) //
// 1 Based!!!!!
static unsigned int PDY2Galep5MSocket[60] =
  {
    24, 23, 22, 21, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10,  9,  8,  7,  6,  5,  4,  3,  2,  1,
    43, 44, 45, 46, 47, 48, 42, 41, 40, 39, 38, 37, 36, 35, 34, 33, 32, 31, 30, 29, 28, 27, 26, 25,
    0,0,0,0,0,0,0,0,0,0,0,0
  };

// Reverse.
// 0 Based!!!!!
static unsigned int G5MSocket2PDY[48] =
  {
    23, 22, 21, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0, 47, 46, 45, 44, 43, 42, 41, 40, 39, 38, 37, 36, 35, 34, 33, 32, 31, 30, 24, 25, 26, 27, 28, 29
  };


cG5LinearSlotHdw::cG5LinearSlotHdw(cG5PDIOCtrl& p)
  : mD(p)
    {
      miSocketMask = 0;
      miDirectionMask = 0;
      miOutputShadow = 0;

      int iUsedPindrivers = mD.cpdCtrl().numUsedPinDrivers();
      for (int i = 0; i < 64-iUsedPindrivers; ++i)
        {
          miSocketMask |= 1;
          miSocketMask <<= 1;
        }
      miSocketMask <<= (iUsedPindrivers-1);
    }

cG5LinearSlotHdw::~cG5LinearSlotHdw()
  { }


unsigned int cG5LinearSlotHdw::pin2Idx(unsigned int iPin)
  {
    if (iPin > 47)
      return 0;
    return G5MSocket2PDY[iPin];
//     unsigned int iIdx = 0;
//     while ((PDY2Galep5MSocket[iIdx] != iPin) && (PDY2Galep5MSocket[iIdx] != 0))
//       iIdx++;
//     if (PDY2Galep5MSocket[iIdx] == iPin)
//       return iIdx;
//     return 0;
  }

/**
  \param iMask  - Each bit represents a pin. (Bit 0 == Pin1)
  \returns Returns the corresponsing latch address.
*/
unsigned long long convertMask(unsigned long long iMask)
  {
    unsigned long long i = 0;
    for (int iPin = 0; iPin < 64; ++iPin)
      {
        unsigned long long d = 1; d <<= iPin;
        if (iMask & d)
          {
            unsigned long long dSocketMask = 1;
            dSocketMask <<= cG5LinearSlotHdw::pin2Idx(iPin);
            i |= dSocketMask;
          }
      }
    return i;
  }

/**
  \param iMask  - Each bit represents a pin. (Bit 0 == Pin1)
  \returns Returns the corresponsing latch address.
*/
unsigned long long fpgaMask2Pin(unsigned long long iMask)
  {
    unsigned long long i = 0;
    for (int iFPGABit = 0; iFPGABit < 64; ++iFPGABit)
      {
        unsigned long long d = 1; d <<= iFPGABit;
        if (iMask & d)
          {
            unsigned long long dPinMask = 1;
            dPinMask <<= (PDY2Galep5MSocket[iFPGABit]-1);
            i |= dPinMask;
          }
      }
    return i;
  }


cG5CPDRegSet& cG5LinearSlotHdw::cpdConfig()
  { return mCPDConfig; }

cG5AbstractCPDCtrl& cG5LinearSlotHdw::cpdCtrl()
  { return mD.cpdCtrl(); }

void cG5LinearSlotHdw::setupCPDs()
  { mD.cpdCtrl().writeRegisterSet(cG5AbstractCPDCtrl::rb_crs0, mCPDConfig); }

unsigned long long cG5LinearSlotHdw::readSocket()
  {
    unsigned short pShort[10];
    for (unsigned int i = 0; i < 10; ++i)
      pShort[i] = mD.fpgaCtrl().read16((0x64)+(i));

    unsigned long long iState = 0;
    iState = pShort[9];
    iState <<= 16;
    iState |= pShort[6];
    iState <<= 16;
    iState |= pShort[3];
    iState <<= 16;
    iState |= pShort[0];

    // Mask out the pins which are not assigned to socket. //
    iState &= ~miSocketMask;
    iState = fpgaMask2Pin(iState);
    return (iState);
  }

volatile unsigned short* cG5LinearSlotHdw::fastOutAccess(unsigned int iPin, unsigned int& iMask)
  {
    unsigned long long i = 1;
    i <<= iPin;
    i = convertMask(i);
    
    iMask = (i & 0xffff);
    if (iMask)
      return mD.fpgaCtrl().mpIOSpace+0x20;
    i >>= 16;
    iMask = (i & 0xffff);
    if (iMask)
      return mD.fpgaCtrl().mpIOSpace+0x24;
    i >>= 16;
    iMask = (i & 0xffff);
    if (iMask)
      return mD.fpgaCtrl().mpIOSpace+0x28;
    i >>= 16;
    iMask = (i & 0xffff);
    return mD.fpgaCtrl().mpIOSpace+0x2c;
  }


volatile unsigned short* cG5LinearSlotHdw::fastInAccess(unsigned int iPin, unsigned int& iMask)
  {
    unsigned long long i = 1;
    i <<= iPin;
    i = convertMask(i);
    
    iMask = (i & 0xffff);
    if (iMask)
      return mD.fpgaCtrl().mpIOSpace+0x64;
    i >>= 16;
    iMask = (i & 0xffff);
    if (iMask)
      return mD.fpgaCtrl().mpIOSpace+0x67;
    i >>= 16;
    iMask = (i & 0xffff);
    if (iMask)
      return mD.fpgaCtrl().mpIOSpace+0x6a;
    i >>= 16;
    iMask = (i & 0xffff);
    return mD.fpgaCtrl().mpIOSpace+0x6d;
  }

void cG5LinearSlotHdw::setOutputMask(unsigned long long i)
  {
    miDirectionMask = i;
    i = convertMask(i);

    mD.fpgaCtrl().write16(0x65, i & 0xffff);
    i >>= 16;
    mD.fpgaCtrl().write16(0x68, i & 0xffff);
    i >>= 16;
    mD.fpgaCtrl().write16(0x6b, i & 0xffff);
    i >>= 16;
    mD.fpgaCtrl().write16(0x6e, i & 0xffff);
  }

void cG5LinearSlotHdw::setPinDirection(int iPin, cG5LinearSlotHdw::Direction dir)
  {
    unsigned long long d = 1; d <<= iPin;
    if (dir == dIn)
      miDirectionMask &= ~d;
    else
      miDirectionMask |= d;
    setOutputMask(miDirectionMask);
  }

void cG5LinearSlotHdw::setOutputData(int iPin, bool bo)
  {
    unsigned long long d = 1; d <<= iPin;
    if (bo)
      miOutputShadow |= d;
    else
      miOutputShadow &= ~d;
    writeSocket(miOutputShadow);
  }


void cG5LinearSlotHdw::writeSocket(unsigned long long i)
  {
    miOutputShadow = i;
    i = convertMask(i);

    mD.fpgaCtrl().write16(0x20, i & 0xffff);
    i >>= 16;
    mD.fpgaCtrl().write16(0x24, i & 0xffff);
    i >>= 16;
    mD.fpgaCtrl().write16(0x28, i & 0xffff);
    i >>= 16;
    mD.fpgaCtrl().write16(0x2c, i & 0xffff);
  }

#if 0
    void setFPGAExpand(unsigned int iSlot, unsigned int iOutputMask, unsigned int iData) const
      {
        {
          unsigned short iExpand0 = iData & 0xffff;
          unsigned short iExpand1 = (iData >> 16) & (0xffff);
          // Expand - data: write lines 0..15 //
          mFPGA.write16(((iSlot+1)*0x100)+0x38, iExpand0);
          // Expand - data: write lines 16 - 19 //
          mFPGA.write16(((iSlot+1)*0x100)+0x3b, iExpand1);
        }

        {
          unsigned short iExpand0 = iOutputMask & 0xffff;
          unsigned short iExpand1 = (iOutputMask >> 16) & (0xffff);
          // Expand - data: write lines 0..15 (EXPAND_BUS_FIXED)
          mFPGA.write16(((iSlot+1)*0x100)+0x39, iExpand0);
          // Expand - data: write lines 16..19 (EXPAND_BUS_FIXED)
          mFPGA.write16(((iSlot+1)*0x100)+0x3c, iExpand1);
        }
      }

    unsigned int fpgaExpand(unsigned int iSlot) const
      {
        unsigned short iExpand0 = 0;
        unsigned short iExpand1 = 0;
        // read lines 0..15 //
        iExpand0 = mFPGA.read16(((iSlot+1)*0x100) + (0x37));
        // read lines 16 - 19 //
        iExpand1 = mFPGA.read16(((iSlot+1)*0x100) + (0x3a));
        unsigned int iReturn = iExpand1; iReturn <<= 16; iReturn |= iExpand0;
        return iReturn;
      }


    void setFPGABus(unsigned int iSlot, unsigned int iDirMask, unsigned int iOutputMask) const
      {
        {
          unsigned int iAddrBase = ((iSlot+1)*0x100) + (0x3f);
          mFPGA.write16(iAddrBase, iDirMask);
        }

        {
          unsigned int iAddrBase = ((iSlot+1)*0x100) + (0x3e);
          mFPGA.write16(iAddrBase, iOutputMask);
        }
      }
#endif
