/**
   Copyright (c) 2005,2006,2007,2008 by Conitec Datasysteme GmbH
   Written 2008 by Peer Georgi <georgi@conitec.com>

   License: GPL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#ifndef __cG5PSCtrlR1_h
  #define __cG5PSCtrlR1_h

#include <string>

class cSPICtrl;

/**
  \brief This class \b not throws any exceptions!
*/
class cG5PSCtrlR1
  {
  class cCalibItem
    {
    public:
      cCalibItem(double dGradient, double dOffset);
      cCalibItem(const cCalibItem&);
      cCalibItem();

      cCalibItem& operator=(const cCalibItem&);

      unsigned int dacValue(unsigned int imV) const;
      bool valid() const;
      void invalidate();

    private:
      double mdGradient;
      double mdOffset;
      bool mboValid;
    };

  public:
    /// \brief Readable channels.
    enum PSReadChannel { rchVIn, rchVCP1, rchVCP2, rchVCP3, rchILS, rchICP1, rchICP2, rchICP3, rchVLS, rchVComp, rchVGate };
    /// \brief Writechannels are adjustable power supplies
    enum PSWriteChannel { wchVLS = 0, wchVCOMP = 1, wchVGATE = 2, wchVCP1 = 3, wchVCP2 = 4, wchVCP3 = 5, wchNUM = 6};
  public:
    cG5PSCtrlR1();
    ~cG5PSCtrlR1() throw();

    void getFwVersion(std::string&) const;

    /// \brief Set the voltage on given DAC value.
    bool sltPSSetVoltageDAC(PSWriteChannel, unsigned int iDAC);
    /// \brief Set the voltage on given voltage value (mV).
    bool sltPSSetVoltage(PSWriteChannel, unsigned int imV);
    /// \brief Get all voltage values as DAC values.
    bool sltPSGetUValuesADC(unsigned int& iUIn, unsigned int& iVCP1, unsigned int& iVCP2, unsigned int& iVCP3, unsigned int& iVLS, unsigned int& iVGate, unsigned int& iVComp) const;
    /// \brief Get all voltage values in milli volt.
    bool sltPSGetUValues(unsigned int& iUIn, unsigned int& iVCP1, unsigned int& iVCP2, unsigned int& iVCP3, unsigned int& iVLS, unsigned int& iVGate, unsigned int& iVComp) const;
    /// \brief Get all current adc values.
    bool sltPSGetIValuesADC(unsigned int& iIGate, unsigned int& iIComp, unsigned int& iILS, unsigned int& iICP1, unsigned int& iICP2, unsigned int& iICP3);
    /// \brief Get all current uA values.
    bool sltPSGetIValues(unsigned int& iIGate, unsigned int& iIComp, unsigned int& iILS, unsigned int& iICP1, unsigned int& iICP2, unsigned int& iICP3);
    /// \brief Write calibration data.
    void sltSetSupplyCalibData(PSWriteChannel, double dGradient, double dOffset);
    /// \brief Enable / disable the discharge circuit.
    void sltSetDischarge(PSWriteChannel, bool);

    /// \brief Store calibration table.
    void storeCalibData();

  private:
    unsigned int calcDACValue(PSWriteChannel ch, unsigned int imV);
    unsigned int calcVoltageFromADC(PSReadChannel ch, unsigned int iADC) const;

    void loadCalibData();

  private:
    std::auto_ptr<cSPICtrl> mpSPICtrl;
    cCalibItem mCalibValues[6];
  };

#endif // __cG5PSCtrlR1_h
