/**
   Copyright (c) 2005,2006,2007,2008 by Conitec Datasysteme GmbH
   Written 2008 by Peer Georgi <georgi@conitec.com>

   License: GPL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include "cG5PSCtrlR1Impl.h"



cG5PSCtrlR1Impl::cG5PSCtrlR1Impl(cG5ProductInfo::ProductType i)
  : cG5AbstractPSCtrl(), mPSCtrl(), miProductType(i)
    { }

cG5PSCtrlR1Impl::~cG5PSCtrlR1Impl() throw()
  { }

/**
  Perform a reset to all power supplies.
*/
bool cG5PSCtrlR1Impl::resetPowerSupplies()
  {
    // << order is important >> //
    setVoltage(cG5AbstractPSCtrl::vcp1, 0);
    setVoltage(cG5AbstractPSCtrl::vcp2, 0);
    setVoltage(cG5AbstractPSCtrl::vcp3, 0);
    setVoltage(cG5AbstractPSCtrl::vgate, 0);
    setVoltage(cG5AbstractPSCtrl::vcomp, 0);
    setVoltage(cG5AbstractPSCtrl::vls, 0);
    setDischarge(cG5AbstractPSCtrl::vcp1, false);
    setDischarge(cG5AbstractPSCtrl::vcp2, false);
    setDischarge(cG5AbstractPSCtrl::vcp3, false);
    return (true);
  }

unsigned int cG5PSCtrlR1Impl::dacBits() const
  { return 8; }


unsigned int cG5PSCtrlR1Impl::getVoltageADC(cG5PSCtrlR1Impl::Supply s, bool& boOk)
  {
    unsigned int iUIn = 0;
    unsigned int iVCP1 = 0;
    unsigned int iVCP2 = 0;
    unsigned int iVCP3 = 0;
    unsigned int iVLS = 0;
    unsigned int iVGate = 0;
    unsigned int iVComp = 0;

    boOk = mPSCtrl.sltPSGetUValuesADC(iUIn, iVCP1, iVCP2, iVCP3, iVLS, iVGate, iVComp);

    switch (s)
      {
        case vls:
         return (iVLS);
        case vgate:
         return (iVGate);
        case vcomp:
         return (iVComp);
        case vcp1:
         return (iVCP1);
        case vcp2:
         return (iVCP2);
        case vcp3:
         return (iVCP3);
      }
    return (0);
  }

/**
  Set the current value of one supply.
*/
bool cG5PSCtrlR1Impl::setVoltage(cG5PSCtrlR1Impl::Supply s, unsigned int imV)
  {
    switch (s)
      {
        case vls:
         return (mPSCtrl.sltPSSetVoltage(cG5PSCtrlR1::wchVLS, imV));
        case vgate:
         return (mPSCtrl.sltPSSetVoltage(cG5PSCtrlR1::wchVGATE, imV));
        case vcomp:
         return (mPSCtrl.sltPSSetVoltage(cG5PSCtrlR1::wchVCOMP, imV));
        case vcp1:
         return (mPSCtrl.sltPSSetVoltage(cG5PSCtrlR1::wchVCP1, imV));
        case vcp2:
         return (mPSCtrl.sltPSSetVoltage(cG5PSCtrlR1::wchVCP2, imV));
        case vcp3:
         return (mPSCtrl.sltPSSetVoltage(cG5PSCtrlR1::wchVCP3, imV));
      }
    return (false);
  }

bool cG5PSCtrlR1Impl::setVoltageDAC(Supply s, unsigned int iDAC)
  {
    switch (s)
      {
        case vls:
         return (mPSCtrl.sltPSSetVoltageDAC(cG5PSCtrlR1::wchVLS, iDAC));
        case vgate:
         return (mPSCtrl.sltPSSetVoltageDAC(cG5PSCtrlR1::wchVGATE, iDAC));
        case vcomp:
         return (mPSCtrl.sltPSSetVoltageDAC(cG5PSCtrlR1::wchVCOMP, iDAC));
        case vcp1:
         return (mPSCtrl.sltPSSetVoltageDAC(cG5PSCtrlR1::wchVCP1, iDAC));
        case vcp2:
         return (mPSCtrl.sltPSSetVoltageDAC(cG5PSCtrlR1::wchVCP2, iDAC));
        case vcp3:
         return (mPSCtrl.sltPSSetVoltageDAC(cG5PSCtrlR1::wchVCP3, iDAC));
      }
    return (false);
  }

void cG5PSCtrlR1Impl::storeCalibration()
  { mPSCtrl.storeCalibData(); }



#include <iostream>
void cG5PSCtrlR1Impl::setSupplyCalibData(cG5PSCtrlR1Impl::Supply s, double dGradient, double dOffset)
  {
    switch (s)
      {
        case vls:
         return (mPSCtrl.sltSetSupplyCalibData(cG5PSCtrlR1::wchVLS, dGradient, dOffset));
        case vgate:
         return (mPSCtrl.sltSetSupplyCalibData(cG5PSCtrlR1::wchVGATE, dGradient, dOffset));
        case vcomp:
         return (mPSCtrl.sltSetSupplyCalibData(cG5PSCtrlR1::wchVCOMP, dGradient, dOffset));
        case vcp1:
         return (mPSCtrl.sltSetSupplyCalibData(cG5PSCtrlR1::wchVCP1, dGradient, dOffset));
        case vcp2:
         return (mPSCtrl.sltSetSupplyCalibData(cG5PSCtrlR1::wchVCP2, dGradient, dOffset));
        case vcp3:
         return (mPSCtrl.sltSetSupplyCalibData(cG5PSCtrlR1::wchVCP3, dGradient, dOffset));
      }
  }

std::string cG5PSCtrlR1Impl::getFwVersion() const
  {
    std::string s;
    mPSCtrl.getFwVersion(s);
    return s;
  }

bool cG5PSCtrlR1Impl::getCurrentDraws(unsigned int& iIGate, unsigned int& iIComp, unsigned int& iILS, unsigned int& iICP1, unsigned int& iICP2, unsigned int& iICP3)
  {
    if (!(mPSCtrl.sltPSGetIValues(iIGate, iIComp, iILS, iICP1, iICP2, iICP3)))
      return false;
    return (true);
  }

bool cG5PSCtrlR1Impl::getVoltages(unsigned int& iVGate, unsigned int& iVComp, unsigned int& iVLS, unsigned int& iVCP1, unsigned int& iVCP2, unsigned int& iVCP3, unsigned int& iUIn)
  {
    iVGate = 0;
    iVComp = 0;
    iVLS = 0;
    if (!(mPSCtrl.sltPSGetUValues(iUIn, iVCP1, iVCP2, iVCP3, iVLS, iVGate, iVComp)))
      return false;
    return true;
  }


bool cG5PSCtrlR1Impl::isVoltageReadable(Supply i)
  {
    // enum Supply         { vls = 0, vgate = 1, vcomp = 2, vcp1 = 3, vcp2 = 4, vcp3 = 5 };
    bool boVReadMapG5M[] = { false,   false,     false,     true,     true,      true };
    bool boVReadMapG5D[] = { true,    false,     false,     true,     true,      true };
    bool boVReadMapG5P[] = { true,    false,     false,     true,     true,      true };
    unsigned int iIdx(i);
    if (iIdx > 5)
      return false;
    switch (miProductType)
      {
        case cG5ProductInfo::Galep5M:
          return boVReadMapG5M[iIdx];
        case cG5ProductInfo::Galep5Pro:
          return boVReadMapG5D[iIdx];
        case cG5ProductInfo::Galep5P:
          return boVReadMapG5P[iIdx];
        case cG5ProductInfo::Unknown:
          return false;
      }
    return false;
  }

bool cG5PSCtrlR1Impl::isCurrentDrawReadable(Supply i)
  {
    // enum Supply         { vls = 0, vgate = 1, vcomp = 2, vcp1 = 3, vcp2 = 4, vcp3 = 5 };
    bool boIReadMapG5M[] = { false,   false,     false,     false,    false,     false };
    bool boIReadMapG5D[] = { true,    false,     false,     true,     true,      true };
    bool boIReadMapG5P[] = { true,    false,     false,     true,     true,      true };
    unsigned int iIdx(i);
    if (iIdx > 5)
      return false;
    switch (miProductType)
      {
        case cG5ProductInfo::Galep5M:
          return boIReadMapG5M[iIdx];
        case cG5ProductInfo::Galep5Pro:
          return boIReadMapG5D[iIdx];
        case cG5ProductInfo::Galep5P:
          return boIReadMapG5P[iIdx];
        case cG5ProductInfo::Unknown:
          return false;
      }
    return false;
  }


bool cG5PSCtrlR1Impl::hasDischarge(Supply i) const
  {
    // enum Supply       { vls = 0, vgate = 1, vcomp = 2, vcp1 = 3, vcp2 = 4, vcp3 = 5 };
    bool boDCCircuit[] = { false,   false,     false,     true,     true,      true };
    unsigned int iIdx(i);
    if (iIdx > 5)
      return false;
    return boDCCircuit[i];
  }

void cG5PSCtrlR1Impl::setDischarge(Supply i, bool bo)
  {
    if (!(hasDischarge(i)))
      throw std::string("No discharge circuit on selected supply!");
    switch (i)
      {
        case vcp1:
         return (mPSCtrl.sltSetDischarge(cG5PSCtrlR1::wchVCP1, bo));
        case vcp2:
         return (mPSCtrl.sltSetDischarge(cG5PSCtrlR1::wchVCP2, bo));
        case vcp3:
         return (mPSCtrl.sltSetDischarge(cG5PSCtrlR1::wchVCP3, bo));
        default:
         break;
      }
  }

void cG5PSCtrlR1Impl::setupMCU(const char* pBf, unsigned int iSize)
  {
  }
