/**
   Copyright (c) 2005,2006,2007,2008 by Conitec Datasysteme GmbH
   Written 2008 by Peer Georgi <georgi@conitec.com>

   License: GPL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#ifndef __cSPICtrl_h
  #define __cSPICtrl_h

#include <cDriverAttribute.h>

#define ATTR_SPI        "/sys/bus/platform/drivers/galep5-ps/spi"
#define ATTR_SPI_RAW    "/sys/bus/platform/drivers/galep5-ps/raw_spi"
#define ATTR_SPI_CS     "/sys/bus/platform/drivers/galep5-ps/select"
#define ATTR_SPI_RESET  "/sys/bus/platform/drivers/galep5-ps/reset"

/**
  \brief Class for accessing the SPI bus used to control the power supplies.
*/
class cSPICtrl
  {
  public:
    cSPICtrl();
    ~cSPICtrl() throw();

    /// \brief Transfers one byte and returns the receives bytes.
    char spiIO(char bIn, bool boCtrlSelect);
    /// \brief Set the state of reset line.
    void setReset(bool);

    /**
      \brief Set the state of chip-select line.
      \note Note that the chip select line will be influenced by \ref{io}.
      If boCtrlSelect will be given with true.
    */
    void setSelect(bool);

  private:
    /// \brief Handle to the RAW SPI file descriptor.
    cDriverAttribute mAttrRAWSPI;
    /// \brief Handle to the common SPI file descriptor.
    cDriverAttribute mAttrSPI;
    /// \brief Handle to the chip-select attribute.
    cDriverAttribute mAttrSPICS;
    /// \brief Handle to the reset-select attribute.
    cDriverAttribute mAttrSPIRST;
  };

#endif // __cSPICtrl_h
