/**
   Copyright (c) 2005,2006,2007,2008 by Conitec Datasysteme GmbH
   Written 2008 by Peer Georgi <georgi@conitec.com>

   License: GPL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include "cG5Config.h"
#include "cG5FirmwareCtrl.h"
#include "cG5EthernetCtrl.h"
#include "cG5USBNetCtrl.h"
#include "cG5ProductInfo.h"

const char gLibGalep5Version[] = "libGalep5-0.9.9";

class cG5Config::cG5ConfigImpl
  {
  public:
    cG5ConfigImpl::cG5ConfigImpl()
      : mProductInfo(), mFirmwareCtrl()
      {
        if (mProductInfo.productType() != cG5ProductInfo::Galep5M)
          mpEthernetCtrl.reset(new cG5EthernetCtrl());
        mpUSBNetCtrl.reset(new cG5USBNetCtrl());
      }

    cG5ConfigImpl::~cG5ConfigImpl() throw()
      { }

  public:
    cG5ProductInfo mProductInfo;
    cG5FirmwareCtrl mFirmwareCtrl;
    std::auto_ptr<cG5AbstractNetCtrl> mpEthernetCtrl;
    std::auto_ptr<cG5AbstractNetCtrl> mpUSBNetCtrl;
  };


cG5Config::cG5Config()
  {
    mpImpl.reset(new cG5ConfigImpl());
  }

cG5Config::~cG5Config() throw()
  { /* auto_ptr destroys implementation */ }

const cG5ProductInfo& cG5Config::productInfo() const
  { return mpImpl->mProductInfo; }

const cG5FirmwareCtrl& cG5Config::firmware() const
  { return mpImpl->mFirmwareCtrl; }

cG5FirmwareCtrl& cG5Config::firmware()
  { return mpImpl->mFirmwareCtrl; }

bool cG5Config::hasEthernet() const
  {
    return (mpImpl->mpEthernetCtrl.get() != 0);
  }

bool cG5Config::hasUSBNet() const
  {
    return (mpImpl->mpUSBNetCtrl.get() != 0);
  }

cG5AbstractNetCtrl& cG5Config::ethernet()
  {
    if (!(mpImpl->mpEthernetCtrl.get()))
      throw std::string("cG5Config: no ethernet present");
    return *mpImpl->mpEthernetCtrl;
  }

const cG5AbstractNetCtrl& cG5Config::ethernet() const
  {
    if (!(mpImpl->mpEthernetCtrl.get()))
      throw std::string("cG5Config: no ethernet present");
    return *mpImpl->mpEthernetCtrl;
  }


cG5AbstractNetCtrl& cG5Config::usbNet()
  {
    if (!(mpImpl->mpUSBNetCtrl.get()))
      throw std::string("cG5Config: no usb-network present");
    return *mpImpl->mpUSBNetCtrl;
  }

const cG5AbstractNetCtrl& cG5Config::usbNet() const
  {
    if (!(mpImpl->mpUSBNetCtrl.get()))
      throw std::string("cG5Config: no usb-network present");
    return *mpImpl->mpUSBNetCtrl;
  }
