/**
   Copyright (c) 2005,2006,2007,2008 by Conitec Datasysteme GmbH
   Written 2008 by Peer Georgi <georgi@conitec.com>

   License: GPL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#ifndef __cG5Config_h
  #define __cG5Config_h

#include <memory>
#include <string>

extern const char gLibGalep5Version[];

class cG5FirmwareCtrl;
class cG5AbstractNetCtrl;
class cG5ProductInfo;
/**
  \brief This class stores the general device settings.
*/
class cG5Config
  {
  public:
    cG5Config();
    ~cG5Config() throw();

    /// \brief Returns fixed product settings.
    const cG5ProductInfo& productInfo() const;

    /// \brief Returns access to network control.
    cG5AbstractNetCtrl& ethernet();
    /// \brief Returns access to network controll.
    const cG5AbstractNetCtrl& ethernet() const;

    /// \brief Returns access to the usb-network control.
    cG5AbstractNetCtrl& usbNet();
    /// \brief Returns access to the usb-network control.
    const cG5AbstractNetCtrl& usbNet() const;

    /// \brief Returns true if ethernet interface is present.
    bool hasEthernet() const;
    /// \brief Returns true if ethernet interface is present.
    bool hasUSBNet() const;

    /// \brief Returns access to the device-firmware.
    const cG5FirmwareCtrl& firmware() const;
    /// \brief Returns access to the device-firmware.
    cG5FirmwareCtrl& firmware();

  private:
    class cG5ConfigImpl;
    std::auto_ptr<cG5ConfigImpl> mpImpl;
  };

#endif // __cG5Config_h
