/**
   Copyright (c) 2005,2006,2007,2008 by Conitec Datasysteme GmbH
   Written 2008 by Peer Georgi <georgi@conitec.com>

   License: GPL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include "cG5FirmwareCtrl.h"
#include <cFileAttribute.h>


cG5FirmwareCtrl::cG5FirmwareCtrl()
  {
  }


cG5FirmwareCtrl::~cG5FirmwareCtrl() throw()
  {
  }

std::string cG5FirmwareCtrl::fixedPartVersion() const
  {
    std::string sResult;
    try {
      sResult = cFileAttribute::getStringAttr("/etc/firmware.fixedpart.version");
    }
    catch(...)
      {
        sResult = "prerelease-unversioned";
      }
    return sResult;
  }

std::string cG5FirmwareCtrl::varPartVersion() const
  {
    std::string sResult;
    try {
      sResult = cFileAttribute::getStringAttr("/etc/cfg/version");
    }
    catch(...)
      {
        sResult = "prerelease-unversioned";
      }
    return sResult;
  }

#include <cDFDevice.h>
#include <cTinyFs.h>
#include <cTinyFsFile.h>
#include <cTinyFsFileAttribute.h>

#include <iostream>


void cG5FirmwareCtrl::updateEntireFirmware(const char* sImage, unsigned int iImageSize)
  {
    const std::string sDevice = "/dev/mtd0";
    // (1) We have to backup the following files:
    //     * mac-address  (galep5p, galep5d)
    //     * eth0-address (galep5p, galep5d)
    //     * eth0-netmask (galep5p, galep5d)
    //     * eth0-gateway (galep5p, galep5d)
    //     * ps-calib     (all)
    std::auto_ptr<cGuardedBuffer> mac_address;
    std::auto_ptr<cGuardedBuffer> eth0_address;
    std::auto_ptr<cGuardedBuffer> eth0_netmask;
    std::auto_ptr<cGuardedBuffer> eth0_gateway;

    std::auto_ptr<cGuardedBuffer> usb0_address;
    std::auto_ptr<cGuardedBuffer> usb0_netmask;
    std::auto_ptr<cGuardedBuffer> usb0_gateway;
    std::auto_ptr<cGuardedBuffer> usb0_dhcpd_nw;

    std::auto_ptr<cGuardedBuffer> serial_number;

    std::auto_ptr<cGuardedBuffer> ps_calib;

    {  // Perform backup //
      cDFDevice  mtdDevice(sDevice);
      cTinyFs tinyfs(mtdDevice);
      if (cTinyFsFile* pFile = tinyfs.hasFile("mac-address"))
        mac_address = pFile->read();
      if (cTinyFsFile* pFile = tinyfs.hasFile("eth0-address"))
        eth0_address = pFile->read();
      if (cTinyFsFile* pFile = tinyfs.hasFile("eth0-gateway"))
        eth0_gateway = pFile->read();
      if (cTinyFsFile* pFile = tinyfs.hasFile("eth0-netmask"))
        eth0_netmask = pFile->read();

      if (cTinyFsFile* pFile = tinyfs.hasFile("usb0-address"))
        usb0_address = pFile->read();
      if (cTinyFsFile* pFile = tinyfs.hasFile("usb0-gateway"))
        usb0_gateway = pFile->read();
      if (cTinyFsFile* pFile = tinyfs.hasFile("usb0-netmask"))
        usb0_netmask = pFile->read();
      if (cTinyFsFile* pFile = tinyfs.hasFile("usb0-dhcpd_nw"))
        usb0_dhcpd_nw = pFile->read();
      if (cTinyFsFile* pFile = tinyfs.hasFile("serial"))
        serial_number = pFile->read();


      if (cTinyFsFile* pFile = tinyfs.hasFile("ps-calib"))
        ps_calib = pFile->read();
    }

   // sanity checks for given image //
   unsigned int iBLSize = 0; // Size of bootloader //
   {
      cDFDevice  mtdDevice(sImage);
      cTinyFs tinyfs(mtdDevice);
      if (ps_calib.get())
        {
          cTinyFsFile* pFile = tinyfs.hasFile("ps-calib");
          if (!(pFile))
            throw std::string("cG5FirmwareCtrl: Image does not provide \"ps-calib\"!");
          else
            if (pFile->maxMediaSize() < ps_calib->size())
              throw std::string("cG5FirmwareCtrl: Image does not provide \"ps-calib\" of size needed to restore the backup!");
        }
      cTinyFsFile& bootLoader = tinyfs.findFile("LdDataFlash");
      iBLSize = bootLoader.size();
   }

   // Write data to mtd //
   {
     cDFDevice  mtdDevice(sDevice);
     std::auto_ptr<cGuardedBuffer> newImage = cFileAttribute::getFileContent(sImage);
     // patch image to get it bootable //
     // make firmware bootable //
     // [
     unsigned int iMagic = 0;
       {
        int j = 0;
        int i = 0;
        int iPageSize = mtdDevice.numSectors();
        while (iPageSize >>= 1)
          j++;
        i = (iBLSize / 512) + 1 + (j << 13) + (mtdDevice.sectorSize() << 17);
        // bootfhig machen //
        iMagic = i;
      }

    unsigned int* pMagicLocation = (unsigned int*)(newImage->data()+0x14);
    *pMagicLocation = iMagic;

    unsigned int iBytesLeft = iImageSize;
    unsigned int iOffset = 0;
    unsigned int iMaxSize = mtdDevice.sectorSize()*20;
    cGuardedBuffer bfRead(iMaxSize);
    while (iBytesLeft)
      {
        unsigned int iBlockSize = iMaxSize;
        if (iBlockSize > iBytesLeft)
          iBlockSize = iBytesLeft;
        mtdDevice.rawRead(iOffset, bfRead.data(), iBlockSize);
        // Write only if data has been changed //
        if (!(memmem(bfRead.data(), iBlockSize, newImage->data()+iOffset, iBlockSize) == bfRead.data()))
          {
            std::cout << " ** writing at offset: " << iOffset << std::endl;
            mtdDevice.rawWrite(iOffset, newImage->data()+iOffset, iBlockSize);
          }
        iOffset += iBlockSize;
        iBytesLeft -= iBlockSize;
      }
    // ]
   }


   { // Restore configuration //
      cDFDevice  mtdDevice(sDevice);
      cTinyFs tinyfs(mtdDevice);
      if (ps_calib.get())
        cTinyFsFileAttribute::setFileContent("ps-calib", *ps_calib, sDevice);

      if (serial_number.get())
        cTinyFsFileAttribute::setFileContent("serial", *serial_number, sDevice);

      if (mac_address.get())
        cTinyFsFileAttribute::setFileContent("mac-address", *mac_address, sDevice);

      if (eth0_address.get())
        cTinyFsFileAttribute::setFileContent("eth0-address", *eth0_address, sDevice);

      if (eth0_netmask.get())
        cTinyFsFileAttribute::setFileContent("eth0-netmask", *eth0_netmask, sDevice);

      if (eth0_gateway.get())
        cTinyFsFileAttribute::setFileContent("eth0-gateway", *eth0_gateway, sDevice);

      if (usb0_address.get())
        cTinyFsFileAttribute::setFileContent("usb0-address", *usb0_address, sDevice);

      if (usb0_netmask.get())
        cTinyFsFileAttribute::setFileContent("usb0-netmask", *usb0_netmask, sDevice);

      if (usb0_gateway.get())
        cTinyFsFileAttribute::setFileContent("usb0-gateway", *usb0_gateway, sDevice);

      if (usb0_dhcpd_nw.get())
        cTinyFsFileAttribute::setFileContent("usb0-dhcpd_nw", *usb0_dhcpd_nw, sDevice);
   }


  }

void cG5FirmwareCtrl::updateVarPart(const char*, unsigned int iSize)
  {
    throw std::string("cG5FirmwareCtrl::updateVarPart - not implemeneted yet!");
  }
