/**
   Copyright (c) 2005,2006,2007,2008 by Conitec Datasysteme GmbH
   Written 2008 by Peer Georgi <georgi@conitec.com>

   License: GPL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#ifndef __cG5ProductInfo_h
  #define __cG5ProductInfo_h

#include <memory>

/// \brief This class keeps fixed information about the product.
class cG5ProductInfo
  {
  public:
    enum ProductType { Unknown = 0, Galep5M = 1, Galep5Pro = 2, Galep5P = 3 };

  public:
    cG5ProductInfo();
    virtual ~cG5ProductInfo() throw();

    /// \brief Returns the product name.
    std::string productName() const;
    /// \brief Returns the product revision. (Hardware Revision)
    unsigned int productRevision() const;
    /// \brief Returns type of device.
    ProductType productType() const;

    /// \brief Returns the product serial number.
    const std::string& serialNumber() const;
    /// \brief Returns the production number.
    const std::string& productionNumber() const;

    /// \brief Returns the number of pindrivers detected.
    unsigned int getNumPDsInstalled() const;
    /// \brief Returns the base address of all fpgas.
    unsigned int fpgaBaseMappingAddr() const;

  private:
    class cG5ProductInfoImpl;
    std::auto_ptr<cG5ProductInfoImpl> mpImpl;
  };

#endif // __cG5ProductInfo_h
