/**
   Copyright (c) 2005,2006,2007,2008 by Conitec Datasysteme GmbH
   Written 2008 by Peer Georgi <georgi@conitec.com>

   License: GPL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include "cDriverAttribute.h"

#include <errno.h>
#include <sstream>
#include <string>

class cFdGuard
  {
  public:
    cFdGuard()
      : miFd(0)
        { }

    ~cFdGuard() throw()
      {
        if (miFd)
          ::close(miFd);
      }

    void setGuard(int i)
      { miFd = i; }

    void release()
      { miFd = 0; }

    int fd() const
      { return (miFd); }

   private:
     int miFd;
  };

cDriverAttribute::cDriverAttribute(const std::string& s)
  : msDeviceName(s), mboInverse(true)
    { }

cDriverAttribute::~cDriverAttribute() throw()
  { }

void cDriverAttribute::setInverse(bool bo)
  { mboInverse = bo; }

bool cDriverAttribute::inverse() const
  { return mboInverse; }

void cDriverAttribute::openAttribute(cFdGuard& rdGuard, OpenMode om) const
  {
    int iOM = 0;
    if ((om == omReadOnly) || (om == omReadWrite))
      iOM = O_RDONLY;
    if ((om == omWriteOnly) || (om == omReadWrite))
      iOM = O_WRONLY;

    int iFd = ::open(msDeviceName.c_str(), iOM);
    if (iFd <= 0)
      {
        int iError = errno;
        std::stringstream ss;
        ss << "cDriverAttribute: unable to open driver attribute for reading ";
        ss << msDeviceName << " (code: " << iError << " / " << strerror(iError) << ")";
        throw ss.str();
      }
    rdGuard.setGuard(iFd);
  }

void cDriverAttribute::write8(char b)
  {
    cFdGuard guard;
    openAttribute(guard, omWriteOnly);
    if (::write(guard.fd(), &b, 1) != 1)
      throw std::string("cDriverAttribute: unable to set driver attribute");
  }

char cDriverAttribute::read8() const
  {
    cFdGuard guard;
    openAttribute(guard, omReadOnly);
    unsigned int iTry = 10;
    int iErrorCode = 0;
    char b = 0;
    while (iTry)
      {
        if (::read(guard.fd(), &b, 1) != 1)
          {
            iErrorCode = errno;
            if ((errno == EAGAIN) || (errno == EINTR))
              {
                iTry--;
                continue;
              }
            else
              {
                // any fatal error //
                iTry = 0;
                break;
              }
          }
        else
          break;
      }
    if (!(iTry))
      {
        std::stringstream ss;
        ss << "cDriverAttribute: unable to get driver attribute (code ";
        ss << iErrorCode << " / " << strerror(iErrorCode) << ")";
        throw ss.str();
      }
    return (b);
  }

void cDriverAttribute::write(const char* pBf, unsigned int i)
  {
    cFdGuard guard;
    openAttribute(guard, omWriteOnly);
    if (::write(guard.fd(), pBf, i) != (signed)i)
      throw std::string("cDriverAttribute: unable to set driver attribute");
  }

void cDriverAttribute::read(char* pBf, unsigned int i) const
  {
    cFdGuard guard;
    openAttribute(guard, omReadOnly);
    if (::read(guard.fd(), pBf, i) != (signed)i)
      throw std::string("cDriverAttribute: unable to get driver attribute");
  }

void cDriverAttribute::set(bool bo)
  { write8 ((bo ^ mboInverse) ? (1) : (0)); }

bool cDriverAttribute::get() const
  { return (((read8() != 0) ? (true) : (false)) ^ mboInverse); }

void cDriverAttribute::setInt(int i)
  {
    std::stringstream ss;
    ss << i;
    write(ss.str().c_str(), ss.str().length());
  }

int cDriverAttribute::getInt() const
  {
    char pBf[] = { 0,0,0,0,0,0,0,0,0,0 };
    read(pBf, sizeof(pBf));
    std::stringstream ss(pBf);
    int i; ss >> i;
    return (i);
  }
