/**
   Copyright (c) 2005,2006,2007,2008 by Conitec Datasysteme GmbH
   Written 2008 by Peer Georgi <georgi@conitec.com>

   License: GPL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#ifndef __cDriverAttribute_h
  #define __cDriverAttribute_h

#include <string>
class cFdGuard;

/**
  \brief This class manages and ecapsulates a driver attribute.
  <b> \c Example </b>
  * \code
  * // .. //
  * cDriverAttribute attrBL("/sys/bus/platform/drivers/lcd-uc1611/backlight");
  * attrBL.setInt(80);
  * // .. //
  * \endcode
*/
class cDriverAttribute
  {
  enum OpenMode { omWriteOnly, omReadOnly, omReadWrite };
  public:
    cDriverAttribute(const std::string& sDeviceName);
    ~cDriverAttribute() throw();

    /// \brief Set's the inverse flag. (Set- and get-operations are inverted if bo == true.)
    void setInverse(bool bo);
    /// \brief Returns the inverse flag.
    bool inverse() const;
    
    void write(const char* pBf, unsigned int iSize);
    void read(char* pDst, unsigned int iSize) const;
    void write8(char);
    char read8() const;
    
    /// \brief Specialized methods for writing to a bool attribute.
    void set(bool);
    /// \brief Specialized methods for reading to a bool attribute.
    bool get() const;
    
    /// \brief Specialized methods for writing to a int attribute.
    void setInt(int);
    /// \brief Specialized methods for reading to a int attribute.
    int getInt() const;
    
  private:
    void openAttribute(cFdGuard&, OpenMode) const;
  
  private:
    std::string msDeviceName;
    bool mboInverse;
  };

#endif // __cDriverAttribute_h
