/**
   Copyright (c) 2005,2006,2007,2008 by Conitec Datasysteme GmbH
   Written 2008 by Peer Georgi <georgi@conitec.com>

   License: GPL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include "cFileAttribute.h"
#include <sstream>
#include <fstream>

std::string cFileAttribute::getStringAttr(const std::string& sAttr)
  {
    std::ifstream is(sAttr.c_str(), std::ios::in);
    if (!(is.is_open())) throw std::string("cFileAttribute: Unable to open device attribute (")+sAttr+std::string(")");
    std::string s;
    try {
      is >> s;
    }
    catch(...)
      {
        throw std::string("cFileAttribute: Type error while reading device attribute (")+sAttr+std::string(")");
      }
    return s;
  }

unsigned int cFileAttribute::getUIntAttr(const std::string& sAttr)
  {
    std::ifstream is(sAttr.c_str(), std::ios::in);
    if (!(is.is_open())) throw std::string("cFileAttribute: Unable to open device attribute (")+sAttr+std::string(")");
    unsigned int i;
    try {
      is >> i;
    }
    catch(...)
      {
        throw std::string("cFileAttribute: Type error while reading device attribute (")+sAttr+std::string(")");
      }
    return i;
  }

unsigned int cFileAttribute::getHexUIntAttr(const std::string& sAttr)
  {
    std::ifstream is(sAttr.c_str(), std::ios::in);
    if (!(is.is_open())) throw std::string("cFileAttribute: Unable to open device attribute (")+sAttr+std::string(")");
    unsigned int i;
    try {
      is >> std::hex >> i;
    }
    catch(...)
      {
        throw std::string("cFileAttribute: Type error while reading device attribute (")+sAttr+std::string(")");
      }
    return i;
  }

#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <cGuardedBuffer.h>

std::auto_ptr<cGuardedBuffer> cFileAttribute::getFileContent(const std::string& sFile)
  {
    int iFd = ::open(sFile.c_str(), O_RDONLY);
    if (iFd <= 0)
      throw std::string("Unable to open file");


    try {
      // find out the file size //
      if (lseek64(iFd, 0, SEEK_END) < 0)
        throw std::string("cFileAttribute: File seek error");

      long long iSize = lseek64(iFd, 0, SEEK_CUR);
      if (iSize < 0)
        throw std::string("cFileAttribute: File seek error");

      if (lseek64(iFd, 0, SEEK_SET) != 0)
        throw std::string("cFileAttribute: File seek error");

      std::auto_ptr<cGuardedBuffer> pBf(new cGuardedBuffer(iSize));
      unsigned int iResult = ::read(iFd, pBf->data(), pBf->size());
      if (iResult != pBf->size())
        throw std::string("cFileAttribute: Fiel read error");
      close(iFd);
      return pBf;
    }
    catch(...)
      {
        ::close(iFd);
        throw;
      }
  }

