/**
   Copyright (c) 2005,2006,2007,2008 by Conitec Datasysteme GmbH
   Written 2008 by Peer Georgi <georgi@conitec.com>

   License: GPL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include "cTinyFs.h"
#include "cDFDevice.h"
#include "cTinyFsFile.h"
#include <sstream>

#ifdef __G5FREEAPI__
#include "TinyFsDef.h"
#else
#include "../../../../../../boot/app/LdDataFlash/TinyFsDef.h"
#endif
/*
#define FILE_TABLE_FLASH_LOCATION             (0x4000)
#define FILE_TABLE_BOOTLOADER_ITEM_LOCATIONS  (FILE_TABLE_FLASH_LOCATION)
//#define FILE_TABLE_MAX_ITEMS                  (128)
#define FILE_TABLE_MAGIC                      (0x12345678)
// copied from LDDataFlash
// This struct describes one file allocation entry in firmware
struct cSetupTableItemEntry
  {
    char msItem[16];                  // name of image inclusive trailing zero //
    unsigned int mDfBTEntryFlags;    // load / execution flags //
    unsigned int mDfBTEntryDFStart;  // start in dataflash //
    unsigned int mDfBTEntryDFSize;   // size of image //
    unsigned int mDfBTEntryMemStart; // location in sdram where the to be load //
  };

#include "../
*/
#include <iostream>


cTinyFs::cTinyFs(cDFDevice& d)
  : mDevice(d), miMaxFATItems(0)
    {
      mFAT = mount();
    }

cTinyFs::~cTinyFs() throw()
  { }


cTinyFsFile* cTinyFs::hasFile(const std::string& sName)
  {
    std::list<cTinyFsFile>& lst(files());

    if (sName.length() >= 2)
      {
        if (sName[0] == '%') // % describes an index //
          {
            try {
              std::string s = sName.substr(1);
              std::stringstream ss(s);
              unsigned int iIdx;
              unsigned int i = 0;
              ss >> iIdx;
              for (std::list<cTinyFsFile>::iterator it = lst.begin(); it != lst.end(); ++it, ++i)
                {
                  if (i == iIdx)
                    {
                      return &(*it);
                    }
                }

            }
            catch (...)
              { } // ignore errors //
          }
      }


    for (std::list<cTinyFsFile>::iterator it = lst.begin(); it != lst.end(); ++it)
      {
        if ((*it).name() == sName)
          return &(*it);
      }
    return 0;
  }

cTinyFsFile& cTinyFs::findFile(const std::string& sName)
  {
    cTinyFsFile* p = hasFile(sName);
    if (!(p))
      throw std::string("file entry not found (\""+sName+"\")");
    return *p;
  }

std::list<cTinyFsFile>& cTinyFs::files()
  { return *mFAT.get(); }

const std::list<cTinyFsFile>& cTinyFs::files() const
  { return *mFAT.get(); }

std::auto_ptr<std::list<cTinyFsFile> > cTinyFs::mount()
  {
    std::auto_ptr<std::list<cTinyFsFile> > lst(new std::list<cTinyFsFile>);
    tDFBlockTableEntry items[1024];
    mDevice.rawRead(FILE_TABLE_FLASH_LOCATION, (char*)items, sizeof(tDFBlockTableEntry)*1024);
    if (items[0].mDfBTEntryFlags != FILE_TABLE_MAGIC)
      throw std::string("invalid image");

    miMaxFATItems = 0;
    for (int i = 1; i < 1024; i++)
      {
        if (!(items[i].msItem[0]))
          break;
//        if (!(items[i].mDfBTEntryDFSize)) // problem with files of size zero ! //
//          break;
        items[i].msItem[15] = 0;
        miMaxFATItems++;

        cTinyFsFile item(*this, items[i].msItem, items[i].mDfBTEntryDFStart, items[i].mDfBTEntryDFSize, items[i].mDfBTEntryFlags, items[i].mDfBTEntryMemStart);
        lst->insert(lst->end(), item);

//        unsigned int iMaxMediaSize = maxFileSize(item, false);
//        iMediaUsage += iMaxMediaSize;
      }

//     if (iMediaUsage != iMediumSize)
//       throw std::string("Internal bug detected (iMediaSize != iMedaMaxUsable)");
    if (miMaxFATItems)
      miMaxFATItems += 2; // start and end item //
//    std::cout << "FAT Items: " << miMaxFATItems << std::endl;
    return lst;
  }

template<typename _t, typename __t> _t _MIN(_t a, __t b)
  { return (a < b) ? (a) : (b); }

void cTinyFs::writeFAT()
  {
    tDFBlockTableEntry items[1024];
    memset(items, 0, sizeof(tDFBlockTableEntry)*1024);
    items[0].mDfBTEntryFlags = FILE_TABLE_MAGIC;
    unsigned int iIdx = 1;
    for (std::list<cTinyFsFile>::const_iterator it = files().begin(); it != files().end(); ++it)
      {
        if (iIdx >= (miMaxFATItems-1)) // the last item must kept blank //
          throw std::string("too many files in FAT");
        tDFBlockTableEntry* pNew = &items[iIdx];

        const std::string& sName = it->name();

        memset(pNew->msItem,0,16);
        if (!(sName.empty()))
          memcpy(pNew->msItem, sName.c_str(), _MIN(sName.length(), 15u));
        pNew->mDfBTEntryDFStart = it->mediaStart();
        pNew->mDfBTEntryMemStart = it->loadAddress();
        pNew->mDfBTEntryDFSize = it->size();
        pNew->mDfBTEntryFlags = it->attributes();
        iIdx++;
      }
    // we have to lead a start and an end item in the fat //
    mDevice.rawWrite(FILE_TABLE_FLASH_LOCATION, (const char*)items, sizeof(tDFBlockTableEntry)*miMaxFATItems);
  }



cDFDevice& cTinyFs::device()
  { return mDevice; }

const cDFDevice& cTinyFs::device() const
  { return mDevice; }


/**
  Returns the maximum filesize of file.
  \note The maximum size not only results on the max-media size!
  We have to check also the boot-ram!
*/
unsigned int cTinyFs::maxFileSize(const cTinyFsFile& file, bool boCheckBootSituation) const
  {
    /*
      Find out the maximum size of file. (Find out where the next file begins).
    */
    unsigned int iMaxFileSize = device().mediumSize() - file.mediaStart();
    bool boFirstRun = true;
    // check maximum size on media //
    for (std::list<cTinyFsFile>::const_iterator it = files().begin(); it != files().end(); ++it)
      {
        if (&(*it) == &file)
          continue;
        if (it->mediaStart() > file.mediaStart())
          {
            long iPossibleSize = it->mediaStart()-file.mediaStart();
            if ((iPossibleSize < (signed)iMaxFileSize) || (boFirstRun))
              iMaxFileSize = iPossibleSize;
            boFirstRun = false;
          }
      }

    /*
      Check Bootload situation.
    */
    if (boCheckBootSituation)
      {
        if (file.attrLoadedByBootLoader())
          {
            for (std::list<cTinyFsFile>::const_iterator it = files().begin(); it != files().end(); ++it)
              {
                if (&(*it) == &file)
                  continue;
                if (it->loadAddress() > file.loadAddress())
                  {
                    long iPossibleSize = it->loadAddress()-file.loadAddress();
                    if (iPossibleSize < (signed)iMaxFileSize)
                      iMaxFileSize = iPossibleSize;
                  }
              }
          }
      }
    return iMaxFileSize;
  }


void cTinyFs::checkIntegrity(bool boBootTime)
  {
    for (std::list<cTinyFsFile>::const_iterator it = files().begin(); it != files().end(); ++it)
      {
        if (it->size() > maxFileSize(*it, boBootTime))
          {
            std::stringstream ss; ss << "Filesystem corrupt: File \"" << it->name() << "\" too big!";
            throw std::string(ss.str());
          }
      }
  }

#include "cGuardedBuffer.h"
#include <cCRC16.h>

void cTinyFs::checkContentIntegrity(const cTinyFsFile& file)
  {
    cGuardedBuffer bf(file.size());
    file.read(bf.data(), bf.size());
    unsigned int iCSBuffer = cCRC16::crc16(bf.data(), bf.size());
    unsigned int iCSContent = file.contentCRC16();
    if (iCSBuffer != iCSContent)
      {
        std::stringstream ss; ss << "CheckSum error: " << std::hex << "0x" << iCSBuffer << " != 0x" << iCSContent << std::dec << "!";
        throw std::string(ss.str());
      }
  }
