/**
   Copyright (c) 2005,2006,2007,2008 by Conitec Datasysteme GmbH
   Written 2008 by Peer Georgi <georgi@conitec.com>

   License: GPL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include "cTinyFsFile.h"
#include "cDFDevice.h"
#include "cTinyFs.h"
#ifdef __G5FREEAPI__
#include "TinyFsDef.h"
#else
#include "../../../../../../boot/app/LdDataFlash/TinyFsDef.h"
#endif

cTinyFsFile::cTinyFsFile(cTinyFs& fs, const std::string& sName, unsigned int iStart, unsigned int iSize, unsigned int iEntryFlags, unsigned int iLoadMemStart)
  : mFs(fs), msName(sName), miMediaStart(iStart), miMediaSize(iSize), miEntryFlags(iEntryFlags), miLoadMemStart(iLoadMemStart)
    { }

cTinyFsFile::~cTinyFsFile() throw()
  {
  }

void cTinyFsFile::clear()
  { miMediaSize = 0; }

void cTinyFsFile::write(cGuardedBuffer& bf)
  { write(bf.data(), bf.size()); }

void cTinyFsFile::write(const char* p, unsigned int iSize)
  {
    if (miMediaSize + iSize >= maxMediaSize())
      throw std::string("no space left for file");
    mFs.device().rawWrite(miMediaStart+miMediaSize, p, iSize);
    miMediaSize += iSize;
  }

void cTinyFsFile::patch(const char* p, unsigned int iSize)
  {
    if (iSize > miMediaSize)
      throw std::string("*** BUG in cTinyFsFile::patch: iSize too big!");
    mFs.device().rawWrite(miMediaStart, p, iSize);
  }

void cTinyFsFile::read(char* p, unsigned int iSize) const
  {
    if (iSize > miMediaSize)
      throw std::string("*** BUG in cTinyFsFile::read: iSize too big!");

    // use device preferred block size //
    unsigned int iBlockSize = mFs.device().preferredBlockSize();
    unsigned int iBytesLeft = iSize;
    unsigned int iOffset = 0;
    while (iBytesLeft)
      {
        unsigned int iBlock = (iBytesLeft < iBlockSize) ? (iBytesLeft) : (iBlockSize);
        mFs.device().rawRead(miMediaStart+iOffset, p, iBlock);
        iBytesLeft -= iBlock;
        iOffset += iBlock;
        p += iBlock;
      }

//    mFs.device().rawRead(miMediaStart, p, iSize);
  }


std::auto_ptr<cGuardedBuffer> cTinyFsFile::read()
  {
    std::auto_ptr<cGuardedBuffer> pBuffer(new cGuardedBuffer(size()));
    pBuffer->fillZero();
    read(pBuffer->data(), pBuffer->size());
    return pBuffer;
  }


unsigned int cTinyFsFile::size() const
  { return (miMediaSize); }

void cTinyFsFile::setName(const std::string& s)
  { msName = s; }

std::string cTinyFsFile::name() const
  { return (msName); }

unsigned int cTinyFsFile::maxMediaSize() const
  { return mFs.maxFileSize(*this); }

unsigned int cTinyFsFile::mediaStart() const
  { return miMediaStart; }

unsigned int cTinyFsFile::loadAddress() const
  { return (miLoadMemStart); }

unsigned int cTinyFsFile::attributes() const
  { return (miEntryFlags); }

bool cTinyFsFile::attrLoadedByBootLoader() const
  { return (miEntryFlags & DF_BTENTRY_FLAG_COPY); }

bool cTinyFsFile::attrExecutedByBootLoader() const
  { return (miEntryFlags & DF_BTENTRY_FLAG_EXEC); }

bool cTinyFsFile::attrBootable() const
  { return (miMediaSize == 0); }

bool cTinyFsFile::attrAlternateBootOption() const
  { return (miEntryFlags & DF_BTENTRY_FLAG_EXEC_ALTERNATE); }

bool cTinyFsFile::attrCompressed() const
  { return (miEntryFlags & DF_BTENTRY_FLAG_COMPRESSED); }


void cTinyFsFile::setContentCRC16(unsigned short iCRC)
  {
    miEntryFlags &= 0xffff;
    unsigned int iTmp = iCRC; iTmp <<= 16;
    miEntryFlags |= iTmp;
  }

unsigned short cTinyFsFile::contentCRC16() const
  {
    unsigned int iCRC = attributes() >> 16;
    return iCRC;
  }


unsigned int cTinyFsFile::percentualUsage() const
  {
    return (miMediaSize*100)/maxMediaSize();
  }

unsigned int cTinyFsFile::bytesLeft() const
  { return (maxMediaSize()-miMediaSize); }


