/**
   Copyright (c) 2005,2006,2007,2008 by Conitec Datasysteme GmbH
   Written 2008 by Peer Georgi <georgi@conitec.com>

   License: GPL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the
    Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include "cTinyFsFileAttribute.h"
#include "cDFDevice.h"
#include "cTinyFs.h"
#include "cTinyFsFile.h"
#include "cGuardedBuffer.h"
#include <cCRC16.h>

bool cTinyFsFileAttribute::tryGetStringAttr(std::string& sReturn, const std::string& sFile, const std::string& sDevice)
  {
    cDFDevice device(sDevice);
    cTinyFs fs(device);
    cTinyFsFile* pItem = fs.hasFile(sFile);
    if (!(pItem)) return false;
    cGuardedBuffer buffer(pItem->size()+1);
    buffer.fillZero();
    pItem->read(buffer.data(), buffer.size()-1);
    sReturn = buffer.data();
    return true;
  }


std::string cTinyFsFileAttribute::getStringAttr(const std::string& sFile, const std::string& sDevice)
  {
    cDFDevice device(sDevice);
    cTinyFs fs(device);
    cTinyFsFile& item = fs.findFile(sFile);
    cGuardedBuffer buffer(item.size()+1);
    buffer.fillZero();
    item.read(buffer.data(), buffer.size()-1);
    std::string sReturn = buffer.data();
    return sReturn;
  }

#define _min(a,b) ((a < b) ? (a) : (b))

void cTinyFsFileAttribute::setStringAttr(const std::string& sFile, const std::string& s, const std::string& sDevice)
  {
    cDFDevice device(sDevice);
    cTinyFs fs(device);
    cTinyFsFile& item = fs.findFile(sFile);
    cGuardedBuffer buffer(item.size());
    memcpy(buffer.data(), s.c_str(), _min(buffer.size(), s.length()));
    item.clear();
    item.write(buffer.data(), buffer.size());
    unsigned short iLastCRC16 = cCRC16::StartValue;
    iLastCRC16 = cCRC16::crc16Part(buffer.data(), buffer.size(), iLastCRC16);
    item.setContentCRC16(iLastCRC16);
    fs.writeFAT();
  }


std::auto_ptr<cGuardedBuffer> cTinyFsFileAttribute::getFileContent(const std::string& sFile, const std::string& sDevice)
  {
    cDFDevice device(sDevice);
    cTinyFs fs(device);
    cTinyFsFile& item = fs.findFile(sFile);
    std::auto_ptr<cGuardedBuffer> pBuffer(new cGuardedBuffer(item.size()+1));
    pBuffer->fillZero();
    item.read(pBuffer->data(), pBuffer->size()-1);
    return pBuffer;
  }

void cTinyFsFileAttribute::setFileContent(const std::string& sFile, const cGuardedBuffer& buffer, const std::string& sDevice)
  {
    cDFDevice device(sDevice);
    cTinyFs fs(device);
    cTinyFsFile& item = fs.findFile(sFile);
    item.clear();
    item.write(buffer.data(), buffer.size());
    unsigned short iLastCRC16 = cCRC16::StartValue;
    iLastCRC16 = cCRC16::crc16Part(buffer.data(), buffer.size(), iLastCRC16);
    item.setContentCRC16(iLastCRC16);
    fs.writeFAT();
  }
